"""
Yellowness Index :math:`Y`
==========================

Defines the *yellowness* index :math:`Y` computation objects:

-   :func:`colour.colorimetry.yellowness_ASTMD1925`: *Yellowness* index
    :math:`YI` computation of given sample *CIE XYZ* tristimulus values using
    *ASTM D1925* method.
-   :func:`colour.colorimetry.yellowness_ASTME313_alternative`: *Yellowness*
    index :math:`YI` computation of given sample *CIE XYZ* tristimulus values
    using the alternative *ASTM E313* method.
-   :func:`colour.colorimetry.yellowness_ASTME313`: *Yellowness*
    index :math:`YI` computation of given sample *CIE XYZ* tristimulus values
    using the recommended *ASTM E313* method.
-   :attr:`colour.YELLOWNESS_METHODS`: Supported *yellowness* computations
    methods.
-   :func:`colour.yellownes`: *Yellowness* :math:`YI` computation using given
    method.

References
----------
-   :cite:`ASTMInternational2015` : ASTM International. (2015). ASTM E313-15e1
    - Standard Practice for Calculating Yellowness and Whiteness Indices from
    Instrumentally Measured Color Coordinates. doi:10.1520/E0313-20
-   :cite:`X-Rite2012a` : X-Rite, & Pantone. (2012). Color iQC and Color iMatch
    Color Calculations Guide.
    https://www.xrite.com/-/media/xrite/files/apps_engineering_techdocuments/\
c/09_color_calculations_en.pdf
"""

from __future__ import annotations

import numpy as np

from colour.hints import (
    Any,
    ArrayLike,
    FloatingOrNDArray,
    Literal,
    Union,
)
from colour.utilities import (
    CaseInsensitiveMapping,
    as_float,
    filter_kwargs,
    from_range_100,
    to_domain_100,
    tsplit,
    validate_method,
)

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "yellowness_ASTMD1925",
    "yellowness_ASTME313_alternative",
    "YELLOWNESS_COEFFICIENTS_ASTME313",
    "yellowness_ASTME313",
    "YELLOWNESS_METHODS",
    "yellowness",
]


def yellowness_ASTMD1925(XYZ: ArrayLike) -> FloatingOrNDArray:
    """
    Return the *yellowness* index :math:`YI` of given sample *CIE XYZ*
    tristimulus values using *ASTM D1925* method.

    ASTM D1925 has been specifically developed for the definition of the
    yellowness of homogeneous, non-fluorescent, almost neutral-transparent,
    white-scattering or opaque plastics as they will be reviewed under daylight
    condition. It can be other materials as well, as long as they fit into this
    description.

    Parameters
    ----------
    XYZ
        *CIE XYZ* tristimulus values of the sample.

    Returns
    -------
    :class:`np.floating` or :class:`numpy.ndarray`
        *Yellowness* :math:`YI`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``XYZ``    | [0, 100]              | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``YI``     | [0, 100]              | [0, 1]        |
    +------------+-----------------------+---------------+

    -   Input *CIE XYZ* tristimulus values must be adapted to
        *CIE Illuminant C*.

    References
    ----------
    :cite:`ASTMInternational2015`, :cite:`X-Rite2012a`

    Examples
    --------
    >>> XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
    >>> yellowness_ASTMD1925(XYZ)  # doctest: +ELLIPSIS
    10.2999999...
    """

    X, Y, Z = tsplit(to_domain_100(XYZ))

    YI = (100 * (1.28 * X - 1.06 * Z)) / Y

    return as_float(from_range_100(YI))


def yellowness_ASTME313_alternative(XYZ: ArrayLike) -> FloatingOrNDArray:
    """
    Return the *yellowness* index :math:`YI` of given sample *CIE XYZ*
    tristimulus values using the alternative *ASTM E313* method.

    In the original form of *Test Method E313*, an alternative equation was
    recommended for a *yellowness* index. In terms of colorimeter readings,
    it was :math:`YI = 100(1 − B/G)` where :math:`B` and :math:`G` are,
    respectively, blue and green colorimeter readings. Its derivation assumed
    that, because of the limitation of the concept to yellow (or blue) colors,
    it was not necessary to take account of variations in the amber or red
    colorimeter reading :math:`A`. This equation is no longer recommended.

    Parameters
    ----------
    XYZ
        *CIE XYZ* tristimulus values of the sample.

    Returns
    -------
    :class:`np.floating` or :class:`numpy.ndarray`
        *Yellowness* :math:`YI`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``XYZ``    | [0, 100]              | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``YI``     | [0, 100]              | [0, 1]        |
    +------------+-----------------------+---------------+

    -   Input *CIE XYZ* tristimulus values must be adapted to
        *CIE Illuminant C*.

    References
    ----------
    :cite:`ASTMInternational2015`, :cite:`X-Rite2012a`

    Examples
    --------
    >>> XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
    >>> yellowness_ASTME313_alternative(XYZ)  # doctest: +ELLIPSIS
    11.0650000...
    """

    _X, Y, Z = tsplit(to_domain_100(XYZ))

    WI = 100 * (1 - (0.847 * Z) / Y)

    return as_float(from_range_100(WI))


YELLOWNESS_COEFFICIENTS_ASTME313: CaseInsensitiveMapping = (
    CaseInsensitiveMapping(
        {
            "CIE 1931 2 Degree Standard Observer": CaseInsensitiveMapping(
                {
                    "C": np.array([1.2769, 1.0592]),
                    "D65": np.array([1.2985, 1.1335]),
                }
            ),
            "CIE 1964 10 Degree Standard Observer": CaseInsensitiveMapping(
                {
                    "C": np.array([1.2871, 1.0781]),
                    "D65": np.array([1.3013, 1.1498]),
                }
            ),
        }
    )
)
YELLOWNESS_COEFFICIENTS_ASTME313.__doc__ = """
Coefficients :math:`C_X` and :math:`C_Z` for the *ASTM E313* *yellowness* index
:math:`YI` computation method.

References
----------
:cite:`ASTMInternational2015`

Aliases:

-   'cie_2_1931': 'CIE 1931 2 Degree Standard Observer'
-   'cie_10_1964': 'CIE 1964 10 Degree Standard Observer'
"""
YELLOWNESS_COEFFICIENTS_ASTME313[
    "cie_2_1931"
] = YELLOWNESS_COEFFICIENTS_ASTME313["CIE 1931 2 Degree Standard Observer"]
YELLOWNESS_COEFFICIENTS_ASTME313[
    "cie_10_1964"
] = YELLOWNESS_COEFFICIENTS_ASTME313["CIE 1964 10 Degree Standard Observer"]


def yellowness_ASTME313(
    XYZ: ArrayLike,
    C_XZ: ArrayLike = YELLOWNESS_COEFFICIENTS_ASTME313[
        "CIE 1931 2 Degree Standard Observer"
    ]["D65"],
) -> FloatingOrNDArray:
    """
    Return the *yellowness* index :math:`YI` of given sample *CIE XYZ*
    tristimulus values using *ASTM E313* method.

    ASTM E313 has successfully been used for a variety of white or near white
    materials. This includes coatings, plastics, textiles.

    Parameters
    ----------
    XYZ
        *CIE XYZ* tristimulus values of the sample.
    C_XZ
        Coefficients :math:`C_X` and :math:`C_Z` for the
        *CIE 1931 2 Degree Standard Observer* and
        *CIE 1964 10 Degree Standard Observer* and *CIE Illuminant C* and
        *CIE Standard Illuminant D65*.

    Returns
    -------
    :class:`np.floating` or :class:`numpy.ndarray`
        *Yellowness* :math:`YI`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``XYZ``    | [0, 100]              | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``YI``     | [0, 100]              | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`ASTMInternational2015`

    Examples
    --------
    >>> XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
    >>> yellowness_ASTME313(XYZ)  # doctest: +ELLIPSIS
    4.3400000...
    """

    X, Y, Z = tsplit(to_domain_100(XYZ))
    C_X, C_Z = tsplit(C_XZ)

    WI = 100 * (C_X * X - C_Z * Z) / Y

    return as_float(from_range_100(WI))


YELLOWNESS_METHODS = CaseInsensitiveMapping(
    {
        "ASTM D1925": yellowness_ASTMD1925,
        "ASTM E313 Alternative": yellowness_ASTME313_alternative,
        "ASTM E313": yellowness_ASTME313,
    }
)
YELLOWNESS_METHODS.__doc__ = """
Supported *yellowness* computation methods.

References
----------
:cite:`ASTMInternational2015`, :cite:`X-Rite2012a`
"""


def yellowness(
    XYZ: ArrayLike,
    method: Union[
        Literal["ASTM D1925", "ASTM E313", "ASTM E313 Alternative"], str
    ] = "ASTM E313",
    **kwargs: Any,
) -> FloatingOrNDArray:
    """
    Return the *yellowness* :math:`W` using given method.

    Parameters
    ----------
    XYZ
        *CIE XYZ* tristimulus values of the sample.
    method
        Computation method.

    Other Parameters
    ----------------
    C_XZ
        {:func:`colour.colorimetry.yellowness_ASTME313`},
        Coefficients :math:`C_X` and :math:`C_Z` for the
        *CIE 1931 2 Degree Standard Observer* and
        *CIE 1964 10 Degree Standard Observer* and *CIE Illuminant C* and
        *CIE Standard Illuminant D65*.

    Returns
    -------
    :class:`np.floating` or :class:`numpy.ndarray`
        *Yellowness* :math:`Y`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``XYZ``    | [0, 100]              | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``YI``     | [0, 100]              | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`ASTMInternational2015`, :cite:`X-Rite2012a`

    Examples
    --------
    >>> XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
    >>> yellowness(XYZ)  # doctest: +ELLIPSIS
    4.3400000...
    >>> yellowness(XYZ, method='ASTM E313 Alternative')  # doctest: +ELLIPSIS
    11.0650000...
    >>> yellowness(XYZ, method='ASTM D1925')  # doctest: +ELLIPSIS
    10.2999999...
    """

    method = validate_method(method, YELLOWNESS_METHODS)

    function = YELLOWNESS_METHODS[method]

    return function(XYZ, **filter_kwargs(function, **kwargs))



"""Showcases *Prismatic* colourspace computations."""

import numpy as np

import colour
from colour.utilities import message_box

message_box('"Prismatic" Colourspace Computations')

RGB = np.array([0.25, 0.50, 0.75])
message_box(
    f'Converting from the "RGB" colourspace to the "Prismatic" colourspace '
    f'given "RGB" values:\n\n\t{RGB}'
)
print(colour.RGB_to_Prismatic(RGB))

print("\n")

Lrgb = np.array([0.7500000, 0.1666667, 0.3333333, 0.5000000])
message_box(
    f'Converting from the "Prismatic" colourspace to the "RGB" colourspace '
    f'given "Lrgb" values:\n\n\t{Lrgb}'
)
print(colour.Prismatic_to_RGB(Lrgb))

print("\n")

message_box(
    f'Applying 50% desaturation in the "Prismatic" colourspace to the given '
    f'"RGB" values:\n\n\t{RGB}'
)
saturation = 0.5
Lrgb = colour.RGB_to_Prismatic(RGB)
Lrgb[..., 1:] = 1.0 / 3.0 + saturation * (Lrgb[..., 1:] - 1.0 / 3.0)
print(colour.Prismatic_to_RGB(Lrgb))



"""
Mesh Volume Computation Helpers
===============================

Defines the helpers objects related to volume computations.
"""

from __future__ import annotations

import numpy as np
from scipy.spatial import Delaunay

from colour.hints import ArrayLike, Floating, NDArray, Optional

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "is_within_mesh_volume",
]


def is_within_mesh_volume(
    points: ArrayLike, mesh: ArrayLike, tolerance: Optional[Floating] = None
) -> NDArray:
    """
    Return whether given points are within given mesh volume using Delaunay
    triangulation.

    Parameters
    ----------
    points
        Points to check if they are within ``mesh`` volume.
    mesh
        Points of the volume used to generate the Delaunay triangulation.
    tolerance
        Tolerance allowed in the inside-triangle check.

    Returns
    -------
    :class:`numpy.ndarray`
        Whether given points are within given mesh volume.

    Examples
    --------
    >>> mesh = np.array(
    ...     [[-1.0, -1.0, 1.0],
    ...       [1.0, -1.0, 1.0],
    ...       [1.0, -1.0, -1.0],
    ...       [-1.0, -1.0, -1.0],
    ...       [0.0, 1.0, 0.0]]
    ... )
    >>> is_within_mesh_volume(np.array([0.0005, 0.0031, 0.0010]), mesh)
    array(True, dtype=bool)
    >>> a = np.array([[0.0005, 0.0031, 0.0010],
    ...               [0.3205, 0.4131, 0.5100]])
    >>> is_within_mesh_volume(a, mesh)
    array([ True, False], dtype=bool)
    """

    triangulation = Delaunay(mesh)

    simplex = triangulation.find_simplex(points, tol=tolerance)
    simplex = np.where(simplex >= 0, True, False)

    return simplex



"""Defines the unit tests for the :mod:`colour.difference.din99` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.difference import delta_E_DIN99

from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestDelta_E_DIN99",
]


class TestDelta_E_DIN99(unittest.TestCase):
    """
    Define :func:`colour.difference.din99.delta_E_DIN99` definition unit
    tests methods.
    """

    def test_delta_E_DIN99(self):
        """Test :func:`colour.difference.din99.delta_E_DIN99` definition."""

        self.assertAlmostEqual(
            delta_E_DIN99(
                np.array([60.25740000, -34.00990000, 36.26770000]),
                np.array([60.46260000, -34.17510000, 39.43870000]),
            ),
            1.177216620111552,
            places=7,
        )

        self.assertAlmostEqual(
            delta_E_DIN99(
                np.array([63.01090000, -31.09610000, -5.86630000]),
                np.array([62.81870000, -29.79460000, -4.08640000]),
            ),
            0.987529977993114,
            places=7,
        )

        self.assertAlmostEqual(
            delta_E_DIN99(
                np.array([35.08310000, -44.11640000, 3.79330000]),
                np.array([35.02320000, -40.07160000, 1.59010000]),
            ),
            1.535894757971742,
            places=7,
        )

        self.assertAlmostEqual(
            delta_E_DIN99(
                np.array([60.25740000, -34.00990000, 36.26770000]),
                np.array([60.46260000, -34.17510000, 39.43870000]),
                textiles=True,
            ),
            1.215652775586509,
            places=7,
        )

        self.assertAlmostEqual(
            delta_E_DIN99(
                np.array([63.01090000, -31.09610000, -5.86630000]),
                np.array([62.81870000, -29.79460000, -4.08640000]),
                textiles=True,
            ),
            1.025997138865984,
            places=7,
        )

        self.assertAlmostEqual(
            delta_E_DIN99(
                np.array([35.08310000, -44.11640000, 3.79330000]),
                np.array([35.02320000, -40.07160000, 1.59010000]),
                textiles=True,
            ),
            1.539922810033725,
            places=7,
        )

    def test_n_dimensional_delta_E_DIN99(self):
        """
        Test :func:`colour.difference.din99.delta_E_DIN99` definition
        n-dimensional arrays support.
        """

        Lab_1 = np.array([60.25740000, -34.00990000, 36.26770000])
        Lab_2 = np.array([60.46260000, -34.17510000, 39.43870000])
        delta_E = delta_E_DIN99(Lab_1, Lab_2)

        Lab_1 = np.tile(Lab_1, (6, 1))
        Lab_2 = np.tile(Lab_2, (6, 1))
        delta_E = np.tile(delta_E, 6)
        np.testing.assert_almost_equal(
            delta_E_DIN99(Lab_1, Lab_2), delta_E, decimal=7
        )

        Lab_1 = np.reshape(Lab_1, (2, 3, 3))
        Lab_2 = np.reshape(Lab_2, (2, 3, 3))
        delta_E = np.reshape(delta_E, (2, 3))
        np.testing.assert_almost_equal(
            delta_E_DIN99(Lab_1, Lab_2), delta_E, decimal=7
        )

    def test_domain_range_scale_delta_E_DIN99(self):
        """
        Test :func:`colour.difference.din99.delta_E_DIN99` definition
        domain and range scale support.
        """

        Lab_1 = np.array([60.25740000, -34.00990000, 36.26770000])
        Lab_2 = np.array([60.46260000, -34.17510000, 39.43870000])
        delta_E = delta_E_DIN99(Lab_1, Lab_2)

        d_r = (("reference", 1), ("1", 0.01), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    delta_E_DIN99(Lab_1 * factor, Lab_2 * factor),
                    delta_E,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_delta_E_DIN99(self):
        """
        Test :func:`colour.difference.din99.delta_E_DIN99` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            Lab_1 = np.array(case)
            Lab_2 = np.array(case)
            delta_E_DIN99(Lab_1, Lab_2)


if __name__ == "__main__":
    unittest.main()



"""
Common Transfer Functions Utilities
===================================

Defines various transfer functions common utilities.
"""

from __future__ import annotations

import numpy as np

from colour.hints import (
    Boolean,
    FloatingOrArrayLike,
    FloatingOrNDArray,
    Integer,
    IntegerOrArrayLike,
    IntegerOrNDArray,
    NDArray,
    Union,
)
from colour.utilities import as_float, as_int, as_float_array, as_int_array

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "CV_range",
    "legal_to_full",
    "full_to_legal",
]


def CV_range(
    bit_depth: Integer = 10, is_legal: Boolean = False, is_int: Boolean = False
) -> NDArray:
    """
    Return the code value :math:`CV` range for given bit depth, range legality
    and representation.

    Parameters
    ----------
    bit_depth
        Bit depth of the code value :math:`CV` range.
    is_legal
        Whether the code value :math:`CV` range is legal.
    is_int
        Whether the code value :math:`CV` range represents integer code values.

    Returns
    -------
    :class:`numpy.ndarray`
        Code value :math:`CV` range.

    Examples
    --------
    >>> CV_range(8, True, True)
    array([ 16, 235])
    >>> CV_range(8, True, False)  # doctest: +ELLIPSIS
    array([ 0.0627451...,  0.9215686...])
    >>> CV_range(10, False, False)
    array([ 0.,  1.])
    """

    if is_legal:
        ranges = np.array([16, 235])
        ranges *= 2 ** (bit_depth - 8)
    else:
        ranges = np.array([0, 2**bit_depth - 1])

    if not is_int:
        ranges = as_float_array(ranges) / (2**bit_depth - 1)

    return ranges


def legal_to_full(
    CV: Union[FloatingOrArrayLike, IntegerOrArrayLike],
    bit_depth: Integer = 10,
    in_int: Boolean = False,
    out_int: Boolean = False,
) -> Union[FloatingOrNDArray, IntegerOrNDArray]:
    """
    Convert given code value :math:`CV` or float equivalent of a code value at
    a given bit depth from legal range (studio swing) to full range
    (full swing).

    Parameters
    ----------
    CV
        Legal range code value :math:`CV` or float equivalent of a code value
        at a given bit depth.
    bit_depth
        Bit depth used for conversion.
    in_int
        Whether to treat the input value as integer code value or float
        equivalent of a code value at a given bit depth.
    out_int
        Whether to return value as integer code value or float equivalent of a
        code value at a given bit depth.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.integer` or :class:`numpy.ndarray`
        Full range code value :math:`CV` or float equivalent of a code value
        at a given bit depth.

    Examples
    --------
    >>> legal_to_full(64 / 1023)
    0.0
    >>> legal_to_full(940 / 1023)
    1.0
    >>> legal_to_full(64 / 1023, out_int=True)
    0
    >>> legal_to_full(940 / 1023, out_int=True)
    1023
    >>> legal_to_full(64, in_int=True)
    0.0
    >>> legal_to_full(940, in_int=True)
    1.0
    >>> legal_to_full(64, in_int=True, out_int=True)
    0
    >>> legal_to_full(940, in_int=True, out_int=True)
    1023
    """

    CV = as_float_array(CV)

    MV = 2**bit_depth - 1

    CV_full = as_int_array(np.round(CV)) if in_int else CV * MV

    B, W = CV_range(bit_depth, True, True)

    CV_full = (CV_full - B) / (W - B)

    if out_int:
        return as_int(np.round(CV_full * MV))
    else:
        return as_float(CV_full)


def full_to_legal(
    CV: Union[FloatingOrArrayLike, IntegerOrArrayLike],
    bit_depth: Integer = 10,
    in_int: Boolean = False,
    out_int: Boolean = False,
) -> Union[FloatingOrNDArray, IntegerOrNDArray]:
    """
    Convert given code value :math:`CV` or float equivalent of a code value at
    a given bit depth from full range (full swing) to legal range
    (studio swing).

    Parameters
    ----------
    CV
        Full range code value :math:`CV` or float equivalent of a code value at
        a given bit depth.
    bit_depth
        Bit depth used for conversion.
    in_int
        Whether to treat the input value as integer code value or float
        equivalent of a code value at a given bit depth.
    out_int
        Whether to return value as integer code value or float equivalent of a
        code value at a given bit depth.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.integer` or :class:`numpy.ndarray`
        Legal range code value :math:`CV` or float equivalent of a code value
        at a given bit depth.

    Examples
    --------
    >>> full_to_legal(0.0)  # doctest: +ELLIPSIS
    0.0625610...
    >>> full_to_legal(1.0)  # doctest: +ELLIPSIS
    0.9188660...
    >>> full_to_legal(0.0, out_int=True)
    64
    >>> full_to_legal(1.0, out_int=True)
    940
    >>> full_to_legal(0, in_int=True)  # doctest: +ELLIPSIS
    0.0625610...
    >>> full_to_legal(1023, in_int=True)  # doctest: +ELLIPSIS
    0.9188660...
    >>> full_to_legal(0, in_int=True, out_int=True)
    64
    >>> full_to_legal(1023, in_int=True, out_int=True)
    940
    """

    CV = as_float_array(CV)

    MV = 2**bit_depth - 1

    CV_legal = as_int_array(np.round(CV / MV)) if in_int else CV

    B, W = CV_range(bit_depth, True, True)

    CV_legal = (W - B) * CV_legal + B

    if out_int:
        return as_int(np.round(CV_legal))
    else:
        return as_float(CV_legal / MV)



# !/usr/bin/env python
"""Defines the unit tests for the :mod:`colour.adaptation.zhai2018` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.adaptation import chromatic_adaptation_Zhai2018
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestChromaticAdaptationZhai2018",
]


class TestChromaticAdaptationZhai2018(unittest.TestCase):
    """
    Define :func:`colour.adaptation.zhai2018.chromatic_adaptation_Zhai2018`
    definition unit tests methods.
    """

    def test_chromatic_adaptation_Zhai2018(self):
        """
        Test :func:`colour.adaptation.zhai2018.chromatic_adaptation_Zhai2018`
        definition.
        """

        np.testing.assert_almost_equal(
            chromatic_adaptation_Zhai2018(
                XYZ_b=np.array([48.900, 43.620, 6.250]),
                XYZ_wb=np.array([109.850, 100, 35.585]),
                XYZ_wd=np.array([95.047, 100, 108.883]),
                D_b=0.9407,
                D_d=0.9800,
                XYZ_wo=np.array([100, 100, 100]),
            ),
            np.array([39.18561644, 42.15461798, 19.23672036]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            chromatic_adaptation_Zhai2018(
                XYZ_b=np.array([48.900, 43.620, 6.250]),
                XYZ_wb=np.array([109.850, 100, 35.585]),
                XYZ_wd=np.array([95.047, 100, 108.883]),
                D_b=0.9407,
                D_d=0.9800,
                XYZ_wo=np.array([100, 100, 100]),
                transform="CAT16",
            ),
            np.array([40.37398343, 43.69426311, 20.51733764]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            chromatic_adaptation_Zhai2018(
                XYZ_b=np.array([52.034, 58.824, 23.703]),
                XYZ_wb=np.array([92.288, 100, 38.775]),
                XYZ_wd=np.array([105.432, 100, 137.392]),
                D_b=0.6709,
                D_d=0.5331,
                XYZ_wo=np.array([97.079, 100, 141.798]),
            ),
            np.array([57.03242915, 58.93434364, 64.76261333]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            chromatic_adaptation_Zhai2018(
                XYZ_b=np.array([52.034, 58.824, 23.703]),
                XYZ_wb=np.array([92.288, 100, 38.775]),
                XYZ_wd=np.array([105.432, 100, 137.392]),
                D_b=0.6709,
                D_d=0.5331,
                XYZ_wo=np.array([97.079, 100, 141.798]),
                transform="CAT16",
            ),
            np.array([56.77130011, 58.81317888, 64.66922808]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            chromatic_adaptation_Zhai2018(
                XYZ_b=np.array([48.900, 43.620, 6.250]),
                XYZ_wb=np.array([109.850, 100, 35.585]),
                XYZ_wd=np.array([95.047, 100, 108.883]),
            ),
            np.array([38.72444735, 42.09232891, 20.05297620]),
            decimal=7,
        )

    def test_n_dimensional_chromatic_adaptation_Zhai2018(self):
        """
        Test :func:`colour.adaptation.zhai2018.chromatic_adaptation_Zhai2018`
        definition n-dimensional arrays support.
        """

        XYZ_b = np.array([48.900, 43.620, 6.250])
        XYZ_wb = np.array([109.850, 100, 35.585])
        XYZ_wd = np.array([95.047, 100, 108.883])
        D_b = 0.9407
        D_d = 0.9800
        XYZ_d = chromatic_adaptation_Zhai2018(XYZ_b, XYZ_wb, XYZ_wd, D_b, D_d)

        XYZ_b = np.tile(XYZ_b, (6, 1))
        XYZ_d = np.tile(XYZ_d, (6, 1))
        np.testing.assert_almost_equal(
            chromatic_adaptation_Zhai2018(XYZ_b, XYZ_wb, XYZ_wd, D_b, D_d),
            XYZ_d,
            decimal=7,
        )

        XYZ_wb = np.tile(XYZ_wb, (6, 1))
        XYZ_wd = np.tile(XYZ_wd, (6, 1))
        D_b = np.tile(D_b, (6, 1))
        D_d = np.tile(D_d, (6, 1))
        np.testing.assert_almost_equal(
            chromatic_adaptation_Zhai2018(XYZ_b, XYZ_wb, XYZ_wd, D_b, D_d),
            XYZ_d,
            decimal=7,
        )

        XYZ_b = np.reshape(XYZ_b, (2, 3, 3))
        XYZ_wb = np.reshape(XYZ_wb, (2, 3, 3))
        XYZ_wd = np.reshape(XYZ_wd, (2, 3, 3))
        D_b = np.reshape(D_b, (2, 3, 1))
        D_d = np.reshape(D_d, (2, 3, 1))
        XYZ_d = np.reshape(XYZ_d, (2, 3, 3))
        np.testing.assert_almost_equal(
            chromatic_adaptation_Zhai2018(XYZ_b, XYZ_wb, XYZ_wd, D_b, D_d),
            XYZ_d,
            decimal=7,
        )

    def test_domain_range_scale_chromatic_adaptation_Zhai2018(self):
        """
        Test :func:`colour.adaptation.zhai2018.chromatic_adaptation_Zhai2018`
        definition domain and range scale support.
        """

        XYZ_b = np.array([48.900, 43.620, 6.250])
        XYZ_wb = np.array([109.850, 100, 35.585])
        XYZ_wd = np.array([95.047, 100, 108.883])
        XYZ_d = chromatic_adaptation_Zhai2018(XYZ_b, XYZ_wb, XYZ_wd)

        d_r = (("reference", 1), ("1", 0.01), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    chromatic_adaptation_Zhai2018(
                        XYZ_b * factor, XYZ_wb * factor, XYZ_wd * factor
                    ),
                    XYZ_d * factor,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_chromatic_adaptation_Zhai2018(self):
        """
        Test :func:`colour.adaptation.zhai2018.chromatic_adaptation_Zhai2018`
        definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            XYZ_b = np.array(case)
            XYZ_wb = np.array(case)
            XYZ_wd = np.array(case)
            D_b = case[0]
            D_d = case[0]
            XYZ_wo = np.array(case)
            chromatic_adaptation_Zhai2018(
                XYZ_b, XYZ_wb, XYZ_wd, D_b, D_d, XYZ_wo
            )


if __name__ == "__main__":
    unittest.main()



"""Defines the unit tests for the :mod:`colour.models.rgb.common` module."""

import numpy as np
import unittest

from colour.models import XYZ_to_sRGB, sRGB_to_XYZ

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestXYZ_to_sRGB",
    "TestsRGB_to_XYZ",
]


class TestXYZ_to_sRGB(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.common.XYZ_to_sRGB` definition unit tests
    methods.
    """

    def test_XYZ_to_sRGB(self):
        """Test :func:`colour.models.rgb.common.XYZ_to_sRGB` definition."""

        np.testing.assert_almost_equal(
            XYZ_to_sRGB(np.array([0.20654008, 0.12197225, 0.05136952])),
            np.array([0.70573936, 0.19248266, 0.22354169]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_sRGB(np.array([0.14222010, 0.23042768, 0.10495772])),
            np.array([0.25847003, 0.58276102, 0.29718877]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_sRGB(
                np.array([0.07818780, 0.06157201, 0.28099326]),
                np.array([0.34570, 0.35850]),
            ),
            np.array([0.09838967, 0.25404426, 0.65130925]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_sRGB(
                np.array([0.00000000, 0.00000000, 0.00000000]),
                np.array([0.44757, 0.40745]),
            ),
            np.array([0.00000000, 0.00000000, 0.00000000]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_sRGB(
                np.array([0.20654008, 0.12197225, 0.05136952]),
                np.array([0.44757, 0.40745]),
                chromatic_adaptation_transform="Bradford",
            ),
            np.array([0.60873814, 0.23259548, 0.43714892]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_sRGB(
                np.array([0.20654008, 0.12197225, 0.05136952]),
                apply_cctf_encoding=False,
            ),
            np.array([0.45620520, 0.03081070, 0.04091953]),
            decimal=7,
        )


class TestsRGB_to_XYZ(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.common.sRGB_to_XYZ` definition unit tests
    methods.
    """

    def test_sRGB_to_XYZ(self):
        """Test :func:`colour.models.rgb.common.sRGB_to_XYZ` definition."""

        np.testing.assert_almost_equal(
            sRGB_to_XYZ(np.array([0.70573936, 0.19248266, 0.22354169])),
            np.array([0.20654290, 0.12197943, 0.05137140]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            sRGB_to_XYZ(np.array([0.25847003, 0.58276102, 0.29718877])),
            np.array([0.14222582, 0.23043727, 0.10496290]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            sRGB_to_XYZ(
                np.array([0.09838967, 0.25404426, 0.65130925]),
                np.array([0.34570, 0.35850]),
            ),
            np.array([0.07819162, 0.06157356, 0.28099475]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            sRGB_to_XYZ(
                np.array([0.00000000, 0.00000000, 0.00000000]),
                np.array([0.44757, 0.40745]),
            ),
            np.array([0.00000000, 0.00000000, 0.00000000]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            sRGB_to_XYZ(
                np.array([0.60873814, 0.23259548, 0.43714892]),
                np.array([0.44757, 0.40745]),
                chromatic_adaptation_transform="Bradford",
            ),
            np.array([0.20654449, 0.12197792, 0.05137030]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            sRGB_to_XYZ(
                np.array([0.45620520, 0.03081070, 0.04091953]),
                apply_cctf_decoding=False,
            ),
            np.array([0.20654291, 0.12197943, 0.05137141]),
            decimal=7,
        )


if __name__ == "__main__":
    unittest.main()



"""
Defines the unit tests for the
:mod:`colour.models.rgb.transfer_functions.common` module.
"""

import numpy as np
import unittest

from colour.models.rgb.transfer_functions import (
    CV_range,
    legal_to_full,
    full_to_legal,
)
from colour.utilities import ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Development"

__all__ = [
    "TestCV_range",
    "TestLegalToFull",
    "TestFullToLegal",
]


class TestCV_range(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.common.CV_range`
    definition unit tests methods.
    """

    def test_CV_range(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.common.CV_range`
        definition.
        """

        np.testing.assert_array_equal(
            CV_range(8, True, True), np.array([16, 235])
        )

        np.testing.assert_array_equal(
            CV_range(8, False, True), np.array([0, 255])
        )

        np.testing.assert_almost_equal(
            CV_range(8, True, False),
            np.array([0.06274510, 0.92156863]),
            decimal=7,
        )

        np.testing.assert_array_equal(
            CV_range(8, False, False), np.array([0, 1])
        )

        np.testing.assert_array_equal(
            CV_range(10, True, True), np.array([64, 940])
        )

        np.testing.assert_array_equal(
            CV_range(10, False, True), np.array([0, 1023])
        )

        np.testing.assert_almost_equal(
            CV_range(10, True, False),
            np.array([0.06256109, 0.91886608]),
            decimal=7,
        )

        np.testing.assert_array_equal(
            CV_range(10, False, False), np.array([0, 1])
        )


class TestLegalToFull(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.common.legal_to_full`
    definition unit tests methods.
    """

    def test_legal_to_full(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.common.legal_to_full`
        definition.
        """

        self.assertAlmostEqual(legal_to_full(64 / 1023), 0.0)

        self.assertAlmostEqual(legal_to_full(940 / 1023), 1.0)

        self.assertAlmostEqual(legal_to_full(64 / 1023, out_int=True), 0)

        self.assertAlmostEqual(legal_to_full(940 / 1023, out_int=True), 1023)

        self.assertAlmostEqual(legal_to_full(64, in_int=True), 0.0)

        self.assertAlmostEqual(legal_to_full(940, in_int=True), 1.0)

        self.assertAlmostEqual(legal_to_full(64, in_int=True, out_int=True), 0)

        self.assertAlmostEqual(
            legal_to_full(940, in_int=True, out_int=True), 1023
        )

    def test_n_dimensional_legal_to_full(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.common.legal_to_full`
        definition n-dimensional arrays support.
        """

        CV_l = 0.918866080156403
        CV_f = legal_to_full(CV_l, 10)

        CV_l = np.tile(CV_l, 6)
        CV_f = np.tile(CV_f, 6)
        np.testing.assert_almost_equal(
            legal_to_full(CV_l, 10), CV_f, decimal=7
        )

        CV_l = np.reshape(CV_l, (2, 3))
        CV_f = np.reshape(CV_f, (2, 3))
        np.testing.assert_almost_equal(
            legal_to_full(CV_l, 10), CV_f, decimal=7
        )

        CV_l = np.reshape(CV_l, (2, 3, 1))
        CV_f = np.reshape(CV_f, (2, 3, 1))
        np.testing.assert_almost_equal(
            legal_to_full(CV_l, 10), CV_f, decimal=7
        )

    @ignore_numpy_errors
    def test_nan_legal_to_full(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.common.legal_to_full`
        definition nan support.
        """

        legal_to_full(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]), 10)


class TestFullToLegal(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.common.full_to_legal`
    definition unit tests methods.
    """

    def test_full_to_legal(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.common.full_to_legal`
        definition.
        """

        self.assertAlmostEqual(full_to_legal(0.0), 0.062561094819159)

        self.assertAlmostEqual(full_to_legal(1.0), 0.918866080156403)

        self.assertAlmostEqual(full_to_legal(0.0, out_int=True), 64)

        self.assertAlmostEqual(full_to_legal(1.0, out_int=True), 940)

        self.assertAlmostEqual(
            full_to_legal(0, in_int=True), 0.062561094819159
        )

        self.assertAlmostEqual(
            full_to_legal(1023, in_int=True), 0.918866080156403
        )

        self.assertAlmostEqual(full_to_legal(0, in_int=True, out_int=True), 64)

        self.assertAlmostEqual(
            full_to_legal(1023, in_int=True, out_int=True), 940
        )

    def test_n_dimensional_full_to_legal(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.common.full_to_legal`
        definition n-dimensional arrays support.
        """

        CF_f = 1.0
        CV_l = full_to_legal(CF_f, 10)

        CF_f = np.tile(CF_f, 6)
        CV_l = np.tile(CV_l, 6)
        np.testing.assert_almost_equal(
            full_to_legal(CF_f, 10), CV_l, decimal=7
        )

        CF_f = np.reshape(CF_f, (2, 3))
        CV_l = np.reshape(CV_l, (2, 3))
        np.testing.assert_almost_equal(
            full_to_legal(CF_f, 10), CV_l, decimal=7
        )

        CF_f = np.reshape(CF_f, (2, 3, 1))
        CV_l = np.reshape(CV_l, (2, 3, 1))
        np.testing.assert_almost_equal(
            full_to_legal(CF_f, 10), CV_l, decimal=7
        )

    @ignore_numpy_errors
    def test_nan_full_to_legal(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.common.full_to_legal`
        definition nan support.
        """

        full_to_legal(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]), 10)


if __name__ == "__main__":
    unittest.main()



"""Defines the unit tests for the :mod:`colour.colorimetry.yellowness` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.colorimetry import (
    yellowness_ASTMD1925,
    yellowness_ASTME313_alternative,
    yellowness_ASTME313,
)
from colour.colorimetry.yellowness import YELLOWNESS_COEFFICIENTS_ASTME313
from colour.colorimetry.yellowness import yellowness
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestYellownessASTMD1925",
    "TestYellownessASTM313Alternative",
    "TestYellownessASTM313",
    "TestYellowness",
]


class TestYellownessASTMD1925(unittest.TestCase):
    """
    Define :func:`colour.colorimetry.yellowness.yellowness_ASTMD1925`
    definition unit tests methods.
    """

    def test_yellowness_ASTMD1925(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness_ASTMD1925`
        definition.
        """

        self.assertAlmostEqual(
            yellowness_ASTMD1925(
                np.array([95.00000000, 100.00000000, 105.00000000])
            ),
            10.299999999999997,
            places=7,
        )

        self.assertAlmostEqual(
            yellowness_ASTMD1925(
                np.array([105.00000000, 100.00000000, 95.00000000])
            ),
            33.700000000000003,
            places=7,
        )

        self.assertAlmostEqual(
            yellowness_ASTMD1925(
                np.array([100.00000000, 100.00000000, 100.00000000])
            ),
            22.0,
            places=7,
        )

    def test_n_dimensional_yellowness_ASTMD1925(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness_ASTMD1925`
        definition n_dimensional arrays support.
        """

        XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
        YI = yellowness_ASTMD1925(XYZ)

        XYZ = np.tile(XYZ, (6, 1))
        YI = np.tile(YI, 6)
        np.testing.assert_almost_equal(
            yellowness_ASTMD1925(XYZ), YI, decimal=7
        )

        XYZ = np.reshape(XYZ, (2, 3, 3))
        YI = np.reshape(YI, (2, 3))
        np.testing.assert_almost_equal(
            yellowness_ASTMD1925(XYZ), YI, decimal=7
        )

    def test_domain_range_scale_yellowness_ASTMD1925(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness_ASTMD1925`
        definition domain and range scale support.
        """

        XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
        YI = 10.299999999999997

        d_r = (("reference", 1), ("1", 0.01), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    yellowness_ASTMD1925(XYZ * factor), YI * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_yellowness_ASTMD1925(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness_ASTMD1925`
        definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            XYZ = np.array(case)
            yellowness_ASTMD1925(XYZ)


class TestYellownessASTM313Alternative(unittest.TestCase):
    """
    Define :func:`colour.colorimetry.yellowness.\
yellowness_ASTME313_alternative` definition unit tests methods.
    """

    def test_yellowness_ASTME313_alternative(self):
        """
        Test :func:`colour.colorimetry.yellowness.\
yellowness_ASTME313_alternative` definition.
        """

        self.assertAlmostEqual(
            yellowness_ASTME313_alternative(
                np.array([95.00000000, 100.00000000, 105.00000000])
            ),
            11.065000000000003,
            places=7,
        )

        self.assertAlmostEqual(
            yellowness_ASTME313_alternative(
                np.array([105.00000000, 100.00000000, 95.00000000])
            ),
            19.534999999999989,
            places=7,
        )

        self.assertAlmostEqual(
            yellowness_ASTME313_alternative(
                np.array([100.00000000, 100.00000000, 100.00000000])
            ),
            15.300000000000002,
            places=7,
        )

    def test_n_dimensional_yellowness_ASTME313_alternative(self):
        """
        Test :func:`colour.colorimetry.yellowness.\
yellowness_ASTME313_alternative` definition n_dimensional arrays support.
        """

        XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
        YI = yellowness_ASTME313_alternative(XYZ)

        XYZ = np.tile(XYZ, (6, 1))
        YI = np.tile(YI, 6)
        np.testing.assert_almost_equal(
            yellowness_ASTME313_alternative(XYZ), YI, decimal=7
        )

        XYZ = np.reshape(XYZ, (2, 3, 3))
        YI = np.reshape(YI, (2, 3))
        np.testing.assert_almost_equal(
            yellowness_ASTME313_alternative(XYZ), YI, decimal=7
        )

    def test_domain_range_scale_yellowness_ASTME313_alternative(self):
        """
        Test :func:`colour.colorimetry.yellowness.\
yellowness_ASTME313_alternative` definition domain and range scale support.
        """

        XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
        YI = 11.065000000000003

        d_r = (("reference", 1), ("1", 0.01), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    yellowness_ASTME313_alternative(XYZ * factor),
                    YI * factor,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_yellowness_ASTME313_alternative(self):
        """
        Test :func:`colour.colorimetry.yellowness.\
yellowness_ASTME313_alternative` definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            XYZ = np.array(case)
            yellowness_ASTME313_alternative(XYZ)


class TestYellownessASTM313(unittest.TestCase):
    """
    Define :func:`colour.colorimetry.yellowness.yellowness_ASTME313`
    definition unit tests methods.
    """

    def test_yellowness_ASTME313(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness_ASTME313`
        definition.
        """

        self.assertAlmostEqual(
            yellowness_ASTME313(
                np.array([95.00000000, 100.00000000, 105.00000000])
            ),
            4.340000000000003,
            places=7,
        )

        self.assertAlmostEqual(
            yellowness_ASTME313(
                np.array([105.00000000, 100.00000000, 95.00000000])
            ),
            28.660000000000011,
            places=7,
        )

        self.assertAlmostEqual(
            yellowness_ASTME313(
                np.array([100.00000000, 100.00000000, 100.00000000])
            ),
            16.500000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            yellowness_ASTME313(
                np.array([95.00000000, 100.00000000, 105.00000000]),
                YELLOWNESS_COEFFICIENTS_ASTME313[
                    "CIE 1931 2 Degree Standard Observer"
                ]["C"],
            ),
            10.089500000000001,
            places=7,
        )

    def test_n_dimensional_yellowness_ASTME313(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness_ASTME313`
        definition n_dimensional arrays support.
        """

        XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
        YI = yellowness_ASTME313(XYZ)

        XYZ = np.tile(XYZ, (6, 1))
        YI = np.tile(YI, 6)
        np.testing.assert_almost_equal(yellowness_ASTME313(XYZ), YI, decimal=7)

        XYZ = np.reshape(XYZ, (2, 3, 3))
        YI = np.reshape(YI, (2, 3))
        np.testing.assert_almost_equal(yellowness_ASTME313(XYZ), YI, decimal=7)

    def test_domain_range_scale_yellowness_ASTME313(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness_ASTME313`
        definition domain and range scale support.
        """

        XYZ = np.array([95.00000000, 100.00000000, 105.00000000])
        YI = 4.340000000000003

        d_r = (("reference", 1), ("1", 0.01), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    yellowness_ASTME313(XYZ * factor), YI * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_yellowness_ASTME313(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness_ASTME313`
        definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            XYZ = np.array(case)
            yellowness_ASTME313(XYZ)


class TestYellowness(unittest.TestCase):
    """
    Define :func:`colour.colorimetry.yellowness.yellowness` definition unit
    tests methods.
    """

    def test_domain_range_scale_yellowness(self):
        """
        Test :func:`colour.colorimetry.yellowness.yellowness` definition
        domain and range scale support.
        """

        XYZ = np.array([95.00000000, 100.00000000, 105.00000000])

        m = ("ASTM D1925", "ASTM E313")
        v = [yellowness(XYZ, method) for method in m]

        d_r = (("reference", 1), ("1", 0.01), ("100", 1))
        for method, value in zip(m, v):
            for scale, factor in d_r:
                with domain_range_scale(scale):
                    np.testing.assert_almost_equal(
                        yellowness(XYZ * factor, method),
                        value * factor,
                        decimal=7,
                    )


if __name__ == "__main__":
    unittest.main()



"""
Digital Cinema Distribution Master (DCDM)
=========================================

Defines the *DCDM* electro-optical transfer function (EOTF) and its
inverse:

-   :func:`colour.models.eotf_inverse_DCDM`
-   :func:`colour.models.eotf_DCDM`

References
----------
-   :cite:`DigitalCinemaInitiatives2007b` : Digital Cinema Initiatives. (2007).
    Digital Cinema System Specification - Version 1.1.
    http://www.dcimovies.com/archives/spec_v1_1/\
DCI_DCinema_System_Spec_v1_1.pdf
"""

from __future__ import annotations

import numpy as np

from colour.algebra import spow
from colour.hints import (
    Boolean,
    FloatingOrArrayLike,
    FloatingOrNDArray,
    IntegerOrArrayLike,
    IntegerOrNDArray,
    Union,
)
from colour.utilities import as_float_array, as_float, as_int

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "eotf_inverse_DCDM",
    "eotf_DCDM",
]


def eotf_inverse_DCDM(
    XYZ: FloatingOrArrayLike, out_int: Boolean = False
) -> Union[FloatingOrNDArray, IntegerOrNDArray]:
    """
    Define the *DCDM* inverse electro-optical transfer function (EOTF).

    Parameters
    ----------
    XYZ
        *CIE XYZ* tristimulus values.
    out_int
        Whether to return value as integer code value or float equivalent of a
        code value at a given bit depth.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.integer` or :class:`numpy.ndarray`
        Non-linear *CIE XYZ'* tristimulus values.

    Warnings
    --------
    *DCDM* is an absolute transfer function.

    Notes
    -----
    -   *DCDM* is an absolute transfer function, thus the domain and range
        values for the *Reference* and *1* scales are only indicative that the
        data is not affected by scale transformations.

    +----------------+-----------------------+---------------+
    | **Domain \\***  | **Scale - Reference** | **Scale - 1** |
    +================+=======================+===============+
    | ``XYZ``        | ``UN``                | ``UN``        |
    +----------------+-----------------------+---------------+

    +----------------+-----------------------+---------------+
    | **Range \\***   | **Scale - Reference** | **Scale - 1** |
    +================+=======================+===============+
    | ``XYZ_p``      | ``UN``                | ``UN``        |
    +----------------+-----------------------+---------------+

    \\* This definition has an output integer switch, thus the domain-range
    scale information is only given for the floating point mode.

    References
    ----------
    :cite:`DigitalCinemaInitiatives2007b`

    Examples
    --------
    >>> eotf_inverse_DCDM(0.18)  # doctest: +ELLIPSIS
    0.1128186...
    >>> eotf_inverse_DCDM(0.18, out_int=True)
    462
    """

    XYZ = as_float_array(XYZ)

    XYZ_p = spow(XYZ / 52.37, 1 / 2.6)

    if out_int:
        return as_int(np.round(4095 * XYZ_p))
    else:
        return as_float(XYZ_p)


def eotf_DCDM(
    XYZ_p: Union[FloatingOrArrayLike, IntegerOrArrayLike],
    in_int: Boolean = False,
) -> FloatingOrNDArray:
    """
    Define the *DCDM* electro-optical transfer function (EOTF).

    Parameters
    ----------
    XYZ_p
        Non-linear *CIE XYZ'* tristimulus values.
    in_int
        Whether to treat the input value as integer code value or float
        equivalent of a code value at a given bit depth.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        *CIE XYZ* tristimulus values.

    Warnings
    --------
    *DCDM* is an absolute transfer function.

    Notes
    -----
    -   *DCDM* is an absolute transfer function, thus the domain and range
        values for the *Reference* and *1* scales are only indicative that the
        data is not affected by scale transformations.

    +----------------+-----------------------+---------------+
    | **Domain \\***  | **Scale - Reference** | **Scale - 1** |
    +================+=======================+===============+
    | ``XYZ_p``      | ``UN``                | ``UN``        |
    +----------------+-----------------------+---------------+

    +----------------+-----------------------+---------------+
    | **Range \\***   | **Scale - Reference** | **Scale - 1** |
    +================+=======================+===============+
    | ``XYZ``        | ``UN``                | ``UN``        |
    +----------------+-----------------------+---------------+

    \\* This definition has an input integer switch, thus the domain-range
    scale information is only given for the floating point mode.

    References
    ----------
    :cite:`DigitalCinemaInitiatives2007b`

    Examples
    --------
    >>> eotf_DCDM(0.11281860951766724)  # doctest: +ELLIPSIS
    0.18...
    >>> eotf_DCDM(462, in_int=True)  # doctest: +ELLIPSIS
    0.18...
    """

    XYZ_p = as_float_array(XYZ_p)

    if in_int:
        XYZ_p = XYZ_p / 4095

    XYZ = 52.37 * spow(XYZ_p, 2.6)

    return as_float(XYZ)



"""
Chromaticity Coordinates of Illuminants
=======================================

Defines the chromaticity coordinates of the illuminants for the
*CIE 1931 2 Degree Standard Observer* and
*CIE 1964 10 Degree Standard Observer*.

The following *CIE* illuminants are available:

-   CIE Standard Illuminant A
-   CIE Illuminant B
-   CIE Illuminant C
-   CIE Illuminant D Series (D50, D55, D60, D65, D75)
-   CIE Illuminant E
-   Illuminants F Series (FL1, FL2, FL3, FL4, FL5, FL6, FL7, FL8, FL9, FL10,
    FL11, FL12, FL3.1, FL3.10, FL3.11, FL3.12, FL3.13, FL3.14, FL3.15, FL3.2,
    FL3.3, FL3.4, FL3.5, FL3.6, FL3.7, FL3.8, FL3.9)
-   High Pressure Discharge Lamps (HP1, HP2, HP3, HP4, HP5)
-   Typical LED illuminants (LED-B1, LED-B2, LED-B3, LED-B4, LED-B5, LED-BH1,
    LED-RGB1, LED-V1, LED-V2)
-   Recommended indoor illuminants ID65 and ID50.

The following *ISO* illuminants are available:

-   ISO 7589 Photographic Daylight
-   ISO 7589 Sensitometric Daylight
-   ISO 7589 Studio Tungsten
-   ISO 7589 Sensitometric Studio Tungsten
-   ISO 7589 Photoflood
-   ISO 7589 Sensitometric Photoflood
-   ISO 7589 Sensitometric Printer

The following other illuminants are available for the
*CIE 1931 2 Degree Standard Observer* only:

- ACES
- Blackmagic Wide Gamut
- DCI-P3
- ICC D50

Illuminants whose chromaticity coordinates are defined at 15 decimal places
have been computed according to practise *ASTM E308-15* method.

References
----------
-   :cite:`BlackmagicDesign2021` : Blackmagic Design. (2021). Blackmagic
    Generation 5 Color Science. https://drive.google.com/file/d/\
1FF5WO2nvI9GEWb4_EntrBoV9ZIuFToZd/view
-   :cite:`CIETC1-482004h` : CIE TC 1-48. (2004). CIE 015:2004 Colorimetry,
    3rd Edition. In CIE 015:2004 Colorimetry, 3rd Edition. Commission
    Internationale de l'Eclairage. ISBN:978-3-901906-33-6
-   :cite:`Carter2018` : Carter, E. C., Schanda, J. D., Hirschler, R., Jost,
    S., Luo, M. R., Melgosa, M., Ohno, Y., Pointer, M. R., Rich, D. C., Vienot,
    F., Whitehead, L., & Wold, J. H. (2018). CIE 015:2018 Colorimetry, 4th
    Edition. International Commission on Illumination. doi:10.25039/TR.015.2018
-   :cite:`DigitalCinemaInitiatives2007b` : Digital Cinema Initiatives. (2007).
    Digital Cinema System Specification - Version 1.1.
    http://www.dcimovies.com/archives/spec_v1_1/\
DCI_DCinema_System_Spec_v1_1.pdf
-   :cite:`InternationalOrganizationforStandardization2002` : International
    Organization for Standardization. (2002). INTERNATIONAL STANDARD ISO
    7589-2002 - Photography - Illuminants for sensitometry - Specifications for
    daylight, incandescent tungsten and printer.
-   :cite:`InternationalColorConsortium2010` : International Color Consortium.
    (2010). Specification ICC.1:2010 (Profile version 4.3.0.0) (pp. 1-130).
    http://www.color.org/specification/ICC1v43_2010-12.pdf
-   :cite:`TheAcademyofMotionPictureArtsandSciences2014q` : The Academy of
    Motion Picture Arts and Sciences, Science and Technology Council, & Academy
    Color Encoding System (ACES) Project Subcommittee. (2014). Technical
    Bulletin TB-2014-004 - Informative Notes on SMPTE ST 2065-1 - Academy Color
    Encoding Specification (ACES) (pp. 1-40). Retrieved December 19, 2014, from
    http://j.mp/TB-2014-004
-   :cite:`Wikipedia2006a` : Wikipedia. (2006). White points of standard
    illuminants. Retrieved February 24, 2014, from
    http://en.wikipedia.org/wiki/Standard_illuminant#\
White_points_of_standard_illuminants
"""

from __future__ import annotations

import numpy as np

from colour.utilities import CaseInsensitiveMapping

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "CCS_ILLUMINANTS_CIE_STANDARD_OBSERVER_2_DEGREE_CIE1931",
    "CCS_ILLUMINANTS_ACES_STANDARD_OBSERVER_2_DEGREE_CIE1931",
    "CCS_ILLUMINANTS_BLACKMAGIC_DESIGN_STANDARD_OBSERVER_2_DEGREE_CIE1931",
    "CCS_ILLUMINANTS_DCI_STANDARD_OBSERVER_2_DEGREE_CIE1931",
    "CCS_ILLUMINANTS_ICC_STANDARD_OBSERVER_2_DEGREE_CIE1931",
    "CCS_ILLUMINANTS_ISO_STANDARD_OBSERVER_2_DEGREE_CIE1931",
    "CCS_ILLUMINANTS_CIE_STANDARD_OBSERVER_10_DEGREE_CIE1964",
    "CCS_ILLUMINANTS_ISO_STANDARD_OBSERVER_10_DEGREE_CIE1964",
    "CCS_ILLUMINANTS",
]

CCS_ILLUMINANTS_CIE_STANDARD_OBSERVER_2_DEGREE_CIE1931: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {
        "A": np.array([0.44758, 0.40745]),
        "B": np.array([0.34842, 0.35161]),
        "C": np.array([0.31006, 0.31616]),
        "D50": np.array([0.34570, 0.35850]),
        "D55": np.array([0.33243, 0.34744]),
        "D60": np.array([0.321616709705268, 0.337619916550817]),
        "D65": np.array([0.31270, 0.32900]),
        "D75": np.array([0.29903, 0.31488]),
        "E": np.array([1 / 3, 1 / 3]),
        "FL1": np.array([0.31310, 0.33710]),
        "FL2": np.array([0.37210, 0.37510]),
        "FL3": np.array([0.40910, 0.39410]),
        "FL4": np.array([0.44020, 0.40310]),
        "FL5": np.array([0.31380, 0.34520]),
        "FL6": np.array([0.37790, 0.38820]),
        "FL7": np.array([0.31290, 0.32920]),
        "FL8": np.array([0.34580, 0.35860]),
        "FL9": np.array([0.37410, 0.37270]),
        "FL10": np.array([0.34580, 0.35880]),
        "FL11": np.array([0.38050, 0.37690]),
        "FL12": np.array([0.43700, 0.40420]),
        "FL3.1": np.array([0.44070, 0.40330]),
        "FL3.2": np.array([0.38080, 0.37340]),
        "FL3.3": np.array([0.31530, 0.34390]),
        "FL3.4": np.array([0.44290, 0.40430]),
        "FL3.5": np.array([0.37490, 0.36720]),
        "FL3.6": np.array([0.34880, 0.36000]),
        "FL3.7": np.array([0.43840, 0.40450]),
        "FL3.8": np.array([0.38200, 0.38320]),
        "FL3.9": np.array([0.34990, 0.35910]),
        "FL3.10": np.array([0.34550, 0.35600]),
        "FL3.11": np.array([0.32450, 0.34340]),
        "FL3.12": np.array([0.43770, 0.40370]),
        "FL3.13": np.array([0.38300, 0.37240]),
        "FL3.14": np.array([0.34470, 0.36090]),
        "FL3.15": np.array([0.31270, 0.32880]),
        "HP1": np.array([0.53300, 0.4150]),
        "HP2": np.array([0.47780, 0.41580]),
        "HP3": np.array([0.43020, 0.40750]),
        "HP4": np.array([0.38120, 0.37970]),
        "HP5": np.array([0.37760, 0.37130]),
        "LED-B1": np.array([0.45600, 0.40780]),
        "LED-B2": np.array([0.43570, 0.40120]),
        "LED-B3": np.array([0.37560, 0.37230]),
        "LED-B4": np.array([0.34220, 0.35020]),
        "LED-B5": np.array([0.31180, 0.32360]),
        "LED-BH1": np.array([0.44740, 0.40660]),
        "LED-RGB1": np.array([0.45570, 0.42110]),
        "LED-V1": np.array([0.45480, 0.40440]),
        "LED-V2": np.array([0.37810, 0.37750]),
        "ID65": np.array([0.310656625403120, 0.330663091836953]),
        "ID50": np.array([0.343211370103531, 0.360207541805137]),
    }
)
"""
Chromaticity coordinates of the *CIE* illuminants for the
*CIE 1931 2 Degree Standard Observer*.

References
----------
:cite:`CIETC1-482004h`, :cite:`Wikipedia2006a`
"""

CCS_ILLUMINANTS_ACES_STANDARD_OBSERVER_2_DEGREE_CIE1931: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {
        "ACES": np.array([0.32168, 0.33767]),
    }
)
"""
Chromaticity coordinates of the *Academy Color Encoding System* (ACES)
illuminants for the *CIE 1931 2 Degree Standard Observer*.

References
----------
:cite:`TheAcademyofMotionPictureArtsandSciences2014q`
"""

CCS_ILLUMINANTS_BLACKMAGIC_DESIGN_STANDARD_OBSERVER_2_DEGREE_CIE1931: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {
        "Blackmagic Wide Gamut": np.array([0.3127170, 0.3290312]),
    }
)
"""
Chromaticity coordinates of the *Blackmagic Design* illuminants for the
*CIE 1931 2 Degree Standard Observer*.

References
----------
:cite:`BlackmagicDesign2021`
"""

CCS_ILLUMINANTS_DCI_STANDARD_OBSERVER_2_DEGREE_CIE1931: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {
        "DCI-P3": np.array([0.31400, 0.35100]),
    }
)
"""
Chromaticity coordinates of the  *DCI* illuminants for the
*CIE 1931 2 Degree Standard Observer*.

References
----------
:cite:`DigitalCinemaInitiatives2007b`
"""

CCS_ILLUMINANTS_ICC_STANDARD_OBSERVER_2_DEGREE_CIE1931: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {"ICC D50": np.array([0.345702914918791, 0.358538596679933])}
)
"""
Chromaticity coordinates of the *ICC* illuminants for the
*CIE 1931 2 Degree Standard Observer*.

References
----------
:cite:`InternationalColorConsortium2010`
"""

CCS_ILLUMINANTS_ISO_STANDARD_OBSERVER_2_DEGREE_CIE1931: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {
        "ISO 7589 Photographic Daylight": np.array(
            [0.332039098470978, 0.347263885596614]
        ),
        "ISO 7589 Sensitometric Daylight": np.array(
            [0.333818313227557, 0.353436231513603]
        ),
        "ISO 7589 Studio Tungsten": np.array(
            [0.430944089109761, 0.403585442674295]
        ),
        "ISO 7589 Sensitometric Studio Tungsten": np.array(
            [0.431418223648390, 0.407471441950342]
        ),
        "ISO 7589 Photoflood": np.array(
            [0.411146015714843, 0.393719378241161]
        ),
        "ISO 7589 Sensitometric Photoflood": np.array(
            [0.412024776908998, 0.398177410548532]
        ),
        "ISO 7589 Sensitometric Printer": np.array(
            [0.412087967973680, 0.421104984758526]
        ),
    }
)
"""
Chromaticity coordinates of the *ISO* illuminants for the
*CIE 1931 2 Degree Standard Observer*.

References
----------
:cite:`InternationalOrganizationforStandardization2002`
"""

CCS_ILLUMINANTS_CIE_STANDARD_OBSERVER_10_DEGREE_CIE1964: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {
        "A": np.array([0.45117, 0.40594]),
        "B": np.array([0.34980, 0.35270]),
        "C": np.array([0.31039, 0.31905]),
        "D50": np.array([0.34773, 0.35952]),
        "D55": np.array([0.33412, 0.34877]),
        "D60": np.array([0.322986926715820, 0.339275732345997]),
        "D65": np.array([0.31382, 0.33100]),
        "D75": np.array([0.29968, 0.31740]),
        "E": np.array([1 / 3, 1 / 3]),
        "FL1": np.array([0.31811, 0.33559]),
        "FL2": np.array([0.37925, 0.36733]),
        "FL3": np.array([0.41761, 0.38324]),
        "FL4": np.array([0.44920, 0.39074]),
        "FL5": np.array([0.31975, 0.34246]),
        "FL6": np.array([0.38660, 0.37847]),
        "FL7": np.array([0.31569, 0.32960]),
        "FL8": np.array([0.34902, 0.35939]),
        "FL9": np.array([0.37829, 0.37045]),
        "FL10": np.array([0.35090, 0.35444]),
        "FL11": np.array([0.38541, 0.37123]),
        "FL12": np.array([0.44256, 0.39717]),
        "FL3.1": np.array([0.449830684010003, 0.390231404321266]),
        "FL3.2": np.array([0.386924116672933, 0.365756034732821]),
        "FL3.3": np.array([0.321176986855865, 0.340501092654981]),
        "FL3.4": np.array([0.448121275113995, 0.397077112142482]),
        "FL3.5": np.array([0.377814166608895, 0.366625766963060]),
        "FL3.6": np.array([0.351976478983504, 0.361094432889677]),
        "FL3.7": np.array([0.444309208810922, 0.396791387314871]),
        "FL3.8": np.array([0.387588931999771, 0.376305569410173]),
        "FL3.9": np.array([0.354688990710449, 0.353445033593383]),
        "FL3.10": np.array([0.349344792334400, 0.354984421140869]),
        "FL3.11": np.array([0.329267975695120, 0.338865386643537]),
        "FL3.12": np.array([0.442252080438001, 0.401220551071252]),
        "FL3.13": np.array([0.386275268780817, 0.374283190950586]),
        "FL3.14": np.array([0.347255078638291, 0.366808242504180]),
        "FL3.15": np.array([0.314613997909246, 0.333377149377113]),
        "HP1": np.array([0.543334600247307, 0.405289298480431]),
        "HP2": np.array([0.482647330648721, 0.410815644179685]),
        "HP3": np.array([0.435560034503954, 0.398801084399711]),
        "HP4": np.array([0.385193641123543, 0.368275479241015]),
        "HP5": np.array([0.380316415606638, 0.366617114797851]),
        "LED-B1": np.array([0.462504966271043, 0.403041801546906]),
        "LED-B2": np.array([0.442119475258745, 0.396633702892576]),
        "LED-B3": np.array([0.380851979328052, 0.368518548904765]),
        "LED-B4": np.array([0.348371362473402, 0.345065503264192]),
        "LED-B5": np.array([0.316916877024753, 0.322060276350364]),
        "LED-BH1": np.array([0.452772610754910, 0.400032462750000]),
        "LED-RGB1": np.array([0.457036370583652, 0.425381348780888]),
        "LED-V1": np.array([0.453602699414564, 0.398199587905174]),
        "LED-V2": np.array([0.377728483834020, 0.374512315539769]),
        "ID65": np.array([0.312074043269908, 0.332660121024630]),
        "ID50": np.array([0.345621427535976, 0.361228962209198]),
    }
)
"""
Chromaticity coordinates of the *CIE* illuminants for the
*CIE 1964 10 Degree Standard Observer*.

References
----------
:cite:`CIETC1-482004h`, :cite:`Wikipedia2006a`
"""

CCS_ILLUMINANTS_ISO_STANDARD_OBSERVER_10_DEGREE_CIE1964: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {
        "ISO 7589 Photographic Daylight": np.array(
            [0.333716908394534, 0.348592494683065]
        ),
        "ISO 7589 Sensitometric Daylight": np.array(
            [0.336125906007630, 0.354997062476417]
        ),
        "ISO 7589 Studio Tungsten": np.array(
            [0.434575926493196, 0.402219691745325]
        ),
        "ISO 7589 Sensitometric Studio Tungsten": np.array(
            [0.435607674215215, 0.406129244796761]
        ),
        "ISO 7589 Photoflood": np.array(
            [0.414144647169611, 0.392458587686395]
        ),
        "ISO 7589 Sensitometric Photoflood": np.array(
            [0.415625819190627, 0.397002292994179]
        ),
        "ISO 7589 Sensitometric Printer": np.array(
            [0.418841052206998, 0.418695130974955]
        ),
    }
)
"""
Chromaticity coordinates of the *ISO* illuminants for the
*CIE 1964 10 Degree Standard Observer*.

References
----------
:cite:`InternationalOrganizationforStandardization2002`
"""

CCS_ILLUMINANTS: CaseInsensitiveMapping = CaseInsensitiveMapping(
    {
        "CIE 1931 2 Degree Standard Observer": CaseInsensitiveMapping(
            CCS_ILLUMINANTS_CIE_STANDARD_OBSERVER_2_DEGREE_CIE1931
        ),
        "CIE 1964 10 Degree Standard Observer": CaseInsensitiveMapping(
            CCS_ILLUMINANTS_CIE_STANDARD_OBSERVER_10_DEGREE_CIE1964
        ),
    }
)
CCS_ILLUMINANTS.__doc__ = """
Chromaticity coordinates of the illuminants.

Warnings
--------
*DCI-P3* illuminant has no associated spectral distribution. *DCI* has no
official reference spectral measurement for this whitepoint. The closest
matching spectral distribution is *Kinoton 75P* projector.

Notes
-----

*CIE Illuminant D Series D60* illuminant chromaticity coordinates were
computed as follows::

    CCT = 6000 * 1.4388 / 1.438
    xy = colour.temperature.CCT_to_xy_CIE_D(CCT)

    sd = colour.sd_CIE_illuminant_D_series(xy)
    colour.XYZ_to_xy(
        colour.sd_to_XYZ(
            sd, colour.MSDS_CMFS[\
'CIE 1964 10 Degree Standard Observer']) / 100.0)

-   *CIE Illuminant D Series D50* illuminant and
    *CIE Standard Illuminant D Series D65* chromaticity coordinates are rounded
    to 4 decimals as given in the typical RGB colourspaces litterature. Their
    chromaticity coordinates as given in :cite:`CIETC1-482004h` are
    (0.34567, 0.35851) and (0.31272, 0.32903) respectively.
-   *CIE* illuminants with chromaticity coordinates not defined in the
    reference :cite:`Wikipedia2006a` have been calculated using their
    correlated colour temperature and
    :func:`colour.temperature.CCT_to_xy_CIE_D`
    :func:`colour.sd_CIE_illuminant_D_series` and / or
    :func:`colour.sd_to_XYZ` definitions.
-   *ICC D50* chromaticity coordinates were computed with
    :func:`colour.XYZ_to_xy` definition from the *CIE XYZ* tristimulus values
    as given by *ICC*: [96.42, 100.00, 82.49].

References
----------
:cite:`CIETC1-482004h`, :cite:`DigitalCinemaInitiatives2007b`,
:cite:`InternationalOrganizationforStandardization2002`,
:cite:`InternationalColorConsortium2010`,
:cite:`TheAcademyofMotionPictureArtsandSciences2014q`, :cite:`Wikipedia2006a`

Aliases:

-   'cie_2_1931': 'CIE 1931 2 Degree Standard Observer'
-   'cie_10_1964': 'CIE 1964 10 Degree Standard Observer'
"""
CCS_ILLUMINANTS["cie_2_1931"] = CCS_ILLUMINANTS[
    "CIE 1931 2 Degree Standard Observer"
]
CCS_ILLUMINANTS["cie_10_1964"] = CCS_ILLUMINANTS[
    "CIE 1964 10 Degree Standard Observer"
]

CCS_ILLUMINANTS["CIE 1931 2 Degree Standard Observer"].update(
    CCS_ILLUMINANTS_ACES_STANDARD_OBSERVER_2_DEGREE_CIE1931
)

CCS_ILLUMINANTS["CIE 1931 2 Degree Standard Observer"].update(
    CCS_ILLUMINANTS_BLACKMAGIC_DESIGN_STANDARD_OBSERVER_2_DEGREE_CIE1931
)

CCS_ILLUMINANTS["CIE 1931 2 Degree Standard Observer"].update(
    CCS_ILLUMINANTS_DCI_STANDARD_OBSERVER_2_DEGREE_CIE1931
)

CCS_ILLUMINANTS["CIE 1931 2 Degree Standard Observer"].update(
    CCS_ILLUMINANTS_ICC_STANDARD_OBSERVER_2_DEGREE_CIE1931
)

CCS_ILLUMINANTS["CIE 1931 2 Degree Standard Observer"].update(
    CCS_ILLUMINANTS_ISO_STANDARD_OBSERVER_2_DEGREE_CIE1931
)

CCS_ILLUMINANTS["CIE 1964 10 Degree Standard Observer"].update(
    CCS_ILLUMINANTS_ISO_STANDARD_OBSERVER_10_DEGREE_CIE1964
)



from .spectrum import (
    SpectralShape,
    SPECTRAL_SHAPE_DEFAULT,
    SpectralDistribution,
    MultiSpectralDistributions,
    reshape_sd,
    reshape_msds,
    sds_and_msds_to_sds,
    sds_and_msds_to_msds,
)
from .blackbody import sd_blackbody, blackbody_spectral_radiance, planck_law
from .cmfs import (
    LMS_ConeFundamentals,
    RGB_ColourMatchingFunctions,
    XYZ_ColourMatchingFunctions,
)
from .datasets import (
    CCS_ILLUMINANTS,
    CCS_LIGHT_SOURCES,
    MSDS_CMFS,
    MSDS_CMFS_LMS,
    MSDS_CMFS_RGB,
    MSDS_CMFS_STANDARD_OBSERVER,
    SDS_BASIS_FUNCTIONS_CIE_ILLUMINANT_D_SERIES,
    SDS_ILLUMINANTS,
    SDS_LEFS,
    SDS_LEFS_PHOTOPIC,
    SDS_LEFS_SCOTOPIC,
    SDS_LIGHT_SOURCES,
    TVS_ILLUMINANTS,
    TVS_ILLUMINANTS_HUNTERLAB,
)
from .generation import sd_constant, sd_zeros, sd_ones
from .generation import msds_constant, msds_zeros, msds_ones
from .generation import SD_GAUSSIAN_METHODS
from .generation import sd_gaussian, sd_gaussian_normal, sd_gaussian_fwhm
from .generation import SD_SINGLE_LED_METHODS
from .generation import sd_single_led, sd_single_led_Ohno2005
from .generation import SD_MULTI_LEDS_METHODS
from .generation import sd_multi_leds, sd_multi_leds_Ohno2005
from .tristimulus_values import SD_TO_XYZ_METHODS, MSDS_TO_XYZ_METHODS
from .tristimulus_values import sd_to_XYZ, msds_to_XYZ
from .tristimulus_values import (
    SPECTRAL_SHAPE_ASTME308,
    handle_spectral_arguments,
    lagrange_coefficients_ASTME2022,
    tristimulus_weighting_factors_ASTME2022,
    adjust_tristimulus_weighting_factors_ASTME308,
    sd_to_XYZ_integration,
    sd_to_XYZ_tristimulus_weighting_factors_ASTME308,
    sd_to_XYZ_ASTME308,
    msds_to_XYZ_integration,
    msds_to_XYZ_ASTME308,
    wavelength_to_XYZ,
)
from .uniformity import spectral_uniformity
from .correction import BANDPASS_CORRECTION_METHODS
from .correction import bandpass_correction
from .correction import bandpass_correction_Stearns1988
from .illuminants import (
    sd_CIE_standard_illuminant_A,
    sd_CIE_illuminant_D_series,
    daylight_locus_function,
)
from .lefs import (
    sd_mesopic_luminous_efficiency_function,
    mesopic_weighting_function,
)
from .lightness import LIGHTNESS_METHODS
from .lightness import lightness
from .lightness import (
    lightness_Glasser1958,
    lightness_Wyszecki1963,
    lightness_CIE1976,
    lightness_Fairchild2010,
    lightness_Fairchild2011,
    lightness_Abebe2017,
)
from .lightness import intermediate_lightness_function_CIE1976
from .luminance import LUMINANCE_METHODS
from .luminance import luminance
from .luminance import (
    luminance_Newhall1943,
    luminance_ASTMD1535,
    luminance_CIE1976,
    luminance_Fairchild2010,
    luminance_Fairchild2011,
    luminance_Abebe2017,
)
from .luminance import intermediate_luminance_function_CIE1976
from .dominant import (
    dominant_wavelength,
    complementary_wavelength,
    excitation_purity,
    colorimetric_purity,
)
from .photometry import luminous_flux, luminous_efficiency, luminous_efficacy
from .transformations import RGB_10_degree_cmfs_to_LMS_10_degree_cmfs
from .transformations import RGB_2_degree_cmfs_to_XYZ_2_degree_cmfs
from .transformations import RGB_10_degree_cmfs_to_XYZ_10_degree_cmfs
from .transformations import LMS_2_degree_cmfs_to_XYZ_2_degree_cmfs
from .transformations import LMS_10_degree_cmfs_to_XYZ_10_degree_cmfs
from .whiteness import WHITENESS_METHODS
from .whiteness import whiteness
from .whiteness import (
    whiteness_Berger1959,
    whiteness_Taube1960,
    whiteness_Stensby1968,
    whiteness_ASTME313,
    whiteness_Ganz1979,
    whiteness_CIE2004,
)
from .yellowness import YELLOWNESS_METHODS
from .yellowness import yellowness
from .yellowness import (
    yellowness_ASTMD1925,
    yellowness_ASTME313_alternative,
    YELLOWNESS_COEFFICIENTS_ASTME313,
    yellowness_ASTME313,
)

__all__ = [
    "SpectralShape",
    "SPECTRAL_SHAPE_DEFAULT",
    "SpectralDistribution",
    "MultiSpectralDistributions",
    "reshape_sd",
    "reshape_msds",
    "sds_and_msds_to_sds",
    "sds_and_msds_to_msds",
]
__all__ += [
    "sd_blackbody",
    "blackbody_spectral_radiance",
    "planck_law",
]
__all__ += [
    "LMS_ConeFundamentals",
    "RGB_ColourMatchingFunctions",
    "XYZ_ColourMatchingFunctions",
]
__all__ += [
    "CCS_ILLUMINANTS",
    "MSDS_CMFS",
    "MSDS_CMFS_LMS",
    "MSDS_CMFS_RGB",
    "MSDS_CMFS_STANDARD_OBSERVER",
    "SDS_BASIS_FUNCTIONS_CIE_ILLUMINANT_D_SERIES",
    "SDS_ILLUMINANTS",
    "SDS_LEFS",
    "SDS_LEFS_PHOTOPIC",
    "SDS_LEFS_SCOTOPIC",
    "TVS_ILLUMINANTS",
    "TVS_ILLUMINANTS_HUNTERLAB",
    "CCS_LIGHT_SOURCES",
    "SDS_LIGHT_SOURCES",
]
__all__ += [
    "sd_constant",
    "sd_zeros",
    "sd_ones",
]
__all__ += [
    "msds_constant",
    "msds_zeros",
    "msds_ones",
]
__all__ += [
    "SD_GAUSSIAN_METHODS",
]
__all__ += [
    "sd_gaussian",
    "sd_gaussian_normal",
    "sd_gaussian_fwhm",
]
__all__ += [
    "SD_SINGLE_LED_METHODS",
]
__all__ += [
    "sd_single_led",
    "sd_single_led_Ohno2005",
]
__all__ += [
    "SD_MULTI_LEDS_METHODS",
]
__all__ += [
    "sd_multi_leds",
    "sd_multi_leds_Ohno2005",
]
__all__ += [
    "SD_TO_XYZ_METHODS",
    "MSDS_TO_XYZ_METHODS",
]
__all__ += [
    "sd_to_XYZ",
    "msds_to_XYZ",
]
__all__ += [
    "SPECTRAL_SHAPE_ASTME308",
    "handle_spectral_arguments",
    "lagrange_coefficients_ASTME2022",
    "tristimulus_weighting_factors_ASTME2022",
    "adjust_tristimulus_weighting_factors_ASTME308",
    "sd_to_XYZ_integration",
    "sd_to_XYZ_tristimulus_weighting_factors_ASTME308",
    "sd_to_XYZ_ASTME308",
    "msds_to_XYZ_integration",
    "msds_to_XYZ_ASTME308",
    "wavelength_to_XYZ",
]
__all__ += [
    "spectral_uniformity",
]
__all__ += [
    "BANDPASS_CORRECTION_METHODS",
]
__all__ += [
    "bandpass_correction",
]
__all__ += [
    "bandpass_correction_Stearns1988",
]
__all__ += [
    "sd_CIE_standard_illuminant_A",
    "sd_CIE_illuminant_D_series",
    "daylight_locus_function",
]
__all__ += [
    "sd_mesopic_luminous_efficiency_function",
    "mesopic_weighting_function",
]
__all__ += [
    "LIGHTNESS_METHODS",
]
__all__ += [
    "lightness",
]
__all__ += [
    "lightness_Glasser1958",
    "lightness_Wyszecki1963",
    "lightness_CIE1976",
    "lightness_Fairchild2010",
    "lightness_Fairchild2011",
    "lightness_Abebe2017",
]
__all__ += [
    "intermediate_lightness_function_CIE1976",
]
__all__ += [
    "LUMINANCE_METHODS",
]
__all__ += [
    "luminance",
]
__all__ += [
    "luminance_Newhall1943",
    "luminance_ASTMD1535",
    "luminance_CIE1976",
    "luminance_Fairchild2010",
    "luminance_Fairchild2011",
    "luminance_Abebe2017",
]
__all__ += [
    "intermediate_luminance_function_CIE1976",
]
__all__ += [
    "dominant_wavelength",
    "complementary_wavelength",
    "excitation_purity",
    "colorimetric_purity",
]
__all__ += [
    "luminous_flux",
    "luminous_efficiency",
    "luminous_efficacy",
]
__all__ += [
    "RGB_10_degree_cmfs_to_LMS_10_degree_cmfs",
]
__all__ += [
    "RGB_2_degree_cmfs_to_XYZ_2_degree_cmfs",
]
__all__ += [
    "RGB_10_degree_cmfs_to_XYZ_10_degree_cmfs",
]
__all__ += [
    "LMS_2_degree_cmfs_to_XYZ_2_degree_cmfs",
]
__all__ += [
    "LMS_10_degree_cmfs_to_XYZ_10_degree_cmfs",
]
__all__ += [
    "WHITENESS_METHODS",
]
__all__ += [
    "whiteness",
]
__all__ += [
    "whiteness_Berger1959",
    "whiteness_Taube1960",
    "whiteness_Stensby1968",
    "whiteness_ASTME313",
    "whiteness_Ganz1979",
    "whiteness_CIE2004",
]
__all__ += [
    "YELLOWNESS_METHODS",
]
__all__ += [
    "yellowness",
]
__all__ += [
    "yellowness_ASTMD1925",
    "yellowness_ASTME313_alternative",
    "YELLOWNESS_COEFFICIENTS_ASTME313",
    "yellowness_ASTME313",
]



"""
Defines the unit tests for the
:mod:`colour.models.rgb.transfer_functions.smpte_240m` module.
"""

import numpy as np
import unittest

from colour.models.rgb.transfer_functions import oetf_SMPTE240M, eotf_SMPTE240M
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestOetf_SMPTE240M",
    "TestEotf_SMPTE240M",
]


class TestOetf_SMPTE240M(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.smpte_240m.\
oetf_SMPTE240M` definition unit tests methods.
    """

    def test_oetf_SMPTE240M(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.smpte_240m.\
oetf_SMPTE240M` definition.
        """

        self.assertAlmostEqual(oetf_SMPTE240M(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            oetf_SMPTE240M(0.02), 0.080000000000000, places=7
        )

        self.assertAlmostEqual(
            oetf_SMPTE240M(0.18), 0.402285796753870, places=7
        )

        self.assertAlmostEqual(oetf_SMPTE240M(1.0), 1.0, places=7)

    def test_n_dimensional_oetf_SMPTE240M(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.smpte_240m.\
oetf_SMPTE240M` definition n-dimensional arrays support.
        """

        L_c = 0.18
        V_c = oetf_SMPTE240M(L_c)

        L_c = np.tile(L_c, 6)
        V_c = np.tile(V_c, 6)
        np.testing.assert_almost_equal(oetf_SMPTE240M(L_c), V_c, decimal=7)

        L_c = np.reshape(L_c, (2, 3))
        V_c = np.reshape(V_c, (2, 3))
        np.testing.assert_almost_equal(oetf_SMPTE240M(L_c), V_c, decimal=7)

        L_c = np.reshape(L_c, (2, 3, 1))
        V_c = np.reshape(V_c, (2, 3, 1))
        np.testing.assert_almost_equal(oetf_SMPTE240M(L_c), V_c, decimal=7)

    def test_domain_range_scale_oetf_SMPTE240M(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.smpte_240m.\
oetf_SMPTE240M` definition domain and range scale support.
        """

        L_c = 0.18
        V_c = oetf_SMPTE240M(L_c)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    oetf_SMPTE240M(L_c * factor), V_c * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_oetf_SMPTE240M(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.smpte_240m.\
oetf_SMPTE240M` definition nan support.
        """

        oetf_SMPTE240M(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestEotf_SMPTE240M(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.smpte_240m.\
eotf_SMPTE240M` definition unit tests methods.
    """

    def test_eotf_SMPTE240M(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.smpte_240m.\
eotf_SMPTE240M` definition.
        """

        self.assertAlmostEqual(eotf_SMPTE240M(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            eotf_SMPTE240M(0.080000000000000), 0.02, places=7
        )

        self.assertAlmostEqual(
            eotf_SMPTE240M(0.402285796753870), 0.18, places=7
        )

        self.assertAlmostEqual(eotf_SMPTE240M(1.0), 1.0, places=7)

    def test_n_dimensional_eotf_SMPTE240M(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.smpte_240m.\
eotf_SMPTE240M` definition n-dimensional arrays support.
        """

        V_r = 0.402285796753870
        L_r = eotf_SMPTE240M(V_r)

        V_r = np.tile(V_r, 6)
        L_r = np.tile(L_r, 6)
        np.testing.assert_almost_equal(eotf_SMPTE240M(V_r), L_r, decimal=7)

        V_r = np.reshape(V_r, (2, 3))
        L_r = np.reshape(L_r, (2, 3))
        np.testing.assert_almost_equal(eotf_SMPTE240M(V_r), L_r, decimal=7)

        V_r = np.reshape(V_r, (2, 3, 1))
        L_r = np.reshape(L_r, (2, 3, 1))
        np.testing.assert_almost_equal(eotf_SMPTE240M(V_r), L_r, decimal=7)

    def test_domain_range_scale_eotf_SMPTE240M(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.smpte_240m.\
eotf_SMPTE240M` definition domain and range scale support.
        """

        V_r = 0.402285796753870
        L_r = eotf_SMPTE240M(V_r)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_SMPTE240M(V_r * factor), L_r * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_eotf_SMPTE240M(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.smpte_240m.\
eotf_SMPTE240M` definition nan support.
        """

        eotf_SMPTE240M(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


if __name__ == "__main__":
    unittest.main()



"""
Sony S-Log Encodings
====================

Defines the *Sony S-Log* log encodings:

-   :func:`colour.models.log_encoding_SLog`
-   :func:`colour.models.log_decoding_SLog`
-   :func:`colour.models.log_encoding_SLog2`
-   :func:`colour.models.log_decoding_SLog2`
-   :func:`colour.models.log_encoding_SLog3`
-   :func:`colour.models.log_decoding_SLog3`

References
----------
-   :cite:`SonyCorporation2012a` : Sony Corporation. (2012). S-Log2 Technical
    Paper (pp. 1-9). https://drive.google.com/file/d/\
1Q1RYri6BaxtYYxX0D4zVD6lAmbwmgikc/view?usp=sharing
-   :cite:`SonyCorporationd` : Sony Corporation. (n.d.). Technical Summary
    for S-Gamut3.Cine/S-Log3 and S-Gamut3/S-Log3 (pp. 1-7).
    http://community.sony.com/sony/attachments/sony/\
large-sensor-camera-F5-F55/12359/2/TechnicalSummary_for_S-Gamut3Cine_S-Gamut3_S-Log3_V1_00.pdf
"""

from __future__ import annotations

import numpy as np

from colour.hints import (
    Boolean,
    FloatingOrArrayLike,
    FloatingOrNDArray,
    Integer,
)
from colour.models.rgb.transfer_functions import full_to_legal, legal_to_full
from colour.utilities import (
    as_float,
    as_float_array,
    domain_range_scale,
    from_range_1,
    to_domain_1,
)

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "log_encoding_SLog",
    "log_decoding_SLog",
    "log_encoding_SLog2",
    "log_decoding_SLog2",
    "log_encoding_SLog3",
    "log_decoding_SLog3",
]


def log_encoding_SLog(
    x: FloatingOrArrayLike,
    bit_depth: Integer = 10,
    out_normalised_code_value: Boolean = True,
    in_reflection: Boolean = True,
) -> FloatingOrNDArray:
    """
    Define the *Sony S-Log* log encoding curve / opto-electronic transfer
    function.

    Parameters
    ----------
    x
        Reflection or :math:`IRE / 100` input light level :math:`x` to a
        camera.
    bit_depth
        Bit depth used for conversion.
    out_normalised_code_value
        Whether the non-linear *Sony S-Log* data :math:`y` is encoded as
        normalised code values.
    in_reflection
        Whether the light level :math:`x` to a camera is reflection.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Non-linear *Sony S-Log* data :math:`y`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``x``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``y``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`SonyCorporation2012a`

    Examples
    --------
    >>> log_encoding_SLog(0.18)  # doctest: +ELLIPSIS
    0.3849708...

    The values of *IRE and CV of S-Log2 @ISO800* table in
    :cite:`SonyCorporation2012a` are obtained as follows:

    >>> x = np.array([0, 18, 90]) / 100
    >>> np.around(log_encoding_SLog(x, 10, False) * 100).astype(np.int)
    array([ 3, 38, 65])
    >>> np.around(log_encoding_SLog(x) * (2 ** 10 - 1)).astype(np.int)
    array([ 90, 394, 636])
    """

    x = to_domain_1(x)

    if in_reflection:
        x = x / 0.9

    y = np.where(
        x >= 0,
        ((0.432699 * np.log10(x + 0.037584) + 0.616596) + 0.03),
        x * 5 + 0.030001222851889303,
    )

    y_cv = full_to_legal(y, bit_depth) if out_normalised_code_value else y

    return as_float(from_range_1(y_cv))


def log_decoding_SLog(
    y: FloatingOrArrayLike,
    bit_depth: Integer = 10,
    in_normalised_code_value: Boolean = True,
    out_reflection: Boolean = True,
) -> FloatingOrNDArray:
    """
    Define the *Sony S-Log* log decoding curve / electro-optical transfer
    function.

    Parameters
    ----------
    y
        Non-linear *Sony S-Log* data :math:`y`.
    bit_depth
        Bit depth used for conversion.
    in_normalised_code_value
        Whether the non-linear *Sony S-Log* data :math:`y` is encoded as
        normalised code values.
    out_reflection
        Whether the light level :math:`x` to a camera is reflection.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Reflection or :math:`IRE / 100` input light level :math:`x` to a
        camera.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``y``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``x``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`SonyCorporation2012a`

    Examples
    --------
    >>> log_decoding_SLog(0.384970815928670)  # doctest: +ELLIPSIS
    0.1...
    """

    y = to_domain_1(y)

    x = legal_to_full(y, bit_depth) if in_normalised_code_value else y

    with domain_range_scale("ignore"):
        x = np.where(
            y >= log_encoding_SLog(0.0, bit_depth, in_normalised_code_value),
            10 ** ((x - 0.616596 - 0.03) / 0.432699) - 0.037584,
            (x - 0.030001222851889303) / 5.0,
        )

    if out_reflection:
        x = x * 0.9

    return as_float(from_range_1(x))


def log_encoding_SLog2(
    x: FloatingOrArrayLike,
    bit_depth: Integer = 10,
    out_normalised_code_value: Boolean = True,
    in_reflection: Boolean = True,
) -> FloatingOrNDArray:
    """
    Define the *Sony S-Log2* log encoding curve / opto-electronic transfer
    function.

    Parameters
    ----------
    x
        Reflection or :math:`IRE / 100` input light level :math:`x` to a
        camera.
    bit_depth
        Bit depth used for conversion.
    out_normalised_code_value
        Whether the non-linear *Sony S-Log2* data :math:`y` is encoded as
        normalised code values.
    in_reflection
        Whether the light level :math:`x` to a camera is reflection.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Non-linear *Sony S-Log2* data :math:`y`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``x``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``y``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`SonyCorporation2012a`

    Examples
    --------
    >>> log_encoding_SLog2(0.18)  # doctest: +ELLIPSIS
    0.3395325...

    The values of *IRE and CV of S-Log2 @ISO800* table in
    :cite:`SonyCorporation2012a` are obtained as follows:

    >>> x = np.array([0, 18, 90]) / 100
    >>> np.around(log_encoding_SLog2(x, 10, False) * 100).astype(np.int)
    array([ 3, 32, 59])
    >>> np.around(log_encoding_SLog2(x) * (2 ** 10 - 1)).astype(np.int)
    array([ 90, 347, 582])
    """

    x = as_float_array(x)

    return log_encoding_SLog(
        x * 155 / 219, bit_depth, out_normalised_code_value, in_reflection
    )


def log_decoding_SLog2(
    y: FloatingOrArrayLike,
    bit_depth: Integer = 10,
    in_normalised_code_value: Boolean = True,
    out_reflection: Boolean = True,
) -> FloatingOrNDArray:
    """
    Define the *Sony S-Log2* log decoding curve / electro-optical transfer
    function.

    Parameters
    ----------
    y
        Non-linear *Sony S-Log2* data :math:`y`.
    bit_depth
        Bit depth used for conversion.
    in_normalised_code_value
        Whether the non-linear *Sony S-Log2* data :math:`y` is encoded as
        normalised code values.
    out_reflection
        Whether the light level :math:`x` to a camera is reflection.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Reflection or :math:`IRE / 100` input light level :math:`x` to a
        camera.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``y``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``x``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`SonyCorporation2012a`

    Examples
    --------
    >>> log_decoding_SLog2(0.339532524633774)  # doctest: +ELLIPSIS
    0.1...
    """

    return (
        219
        * log_decoding_SLog(
            y, bit_depth, in_normalised_code_value, out_reflection
        )
        / 155
    )


def log_encoding_SLog3(
    x: FloatingOrArrayLike,
    bit_depth: Integer = 10,
    out_normalised_code_value: Boolean = True,
    in_reflection: Boolean = True,
) -> FloatingOrNDArray:
    """
    Define the *Sony S-Log3* log encoding curve / opto-electronic transfer
    function.

    Parameters
    ----------
    x
        Reflection or :math:`IRE / 100` input light level :math:`x` to a
        camera.
    bit_depth
        Bit depth used for conversion.
    out_normalised_code_value
        Whether the non-linear *Sony S-Log3* data :math:`y` is encoded as
        normalised code values.
    in_reflection
        Whether the light level :math:`x` to a camera is reflection.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Non-linear *Sony S-Log3* data :math:`y`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``x``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``y``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`SonyCorporationd`

    Examples
    --------
    >>> log_encoding_SLog3(0.18)  # doctest: +ELLIPSIS
    0.4105571...

    The values of *S-Log3 10bit code values (18%, 90%)* table in
    :cite:`SonyCorporationd` are obtained as follows:

    >>> x = np.array([0, 18, 90]) / 100
    >>> np.around(log_encoding_SLog3(x, 10, False) * 100).astype(np.int)
    array([ 4, 41, 61])
    >>> np.around(log_encoding_SLog3(x) * (2 ** 10 - 1)).astype(np.int)
    array([ 95, 420, 598])
    """

    x = to_domain_1(x)

    if not in_reflection:
        x = x * 0.9

    y = np.where(
        x >= 0.01125000,
        (420 + np.log10((x + 0.01) / (0.18 + 0.01)) * 261.5) / 1023,
        (x * (171.2102946929 - 95) / 0.01125000 + 95) / 1023,
    )

    y_cv = y if out_normalised_code_value else legal_to_full(y, bit_depth)

    return as_float(from_range_1(y_cv))


def log_decoding_SLog3(
    y: FloatingOrArrayLike,
    bit_depth: Integer = 10,
    in_normalised_code_value: Boolean = True,
    out_reflection: Boolean = True,
) -> FloatingOrNDArray:
    """
    Define the *Sony S-Log3* log decoding curve / electro-optical transfer
    function.

    Parameters
    ----------
    y
        Non-linear *Sony S-Log3* data :math:`y`.
    bit_depth
        Bit depth used for conversion.
    in_normalised_code_value
        Whether the non-linear *Sony S-Log3* data :math:`y` is encoded as
        normalised code values.
    out_reflection
        Whether the light level :math:`x` to a camera is reflection.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Reflection or :math:`IRE / 100` input light level :math:`x` to a
        camera.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``y``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``x``      | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`SonyCorporationd`

    Examples
    --------
    >>> log_decoding_SLog3(0.410557184750733)  # doctest: +ELLIPSIS
    0.1...
    """

    y = to_domain_1(y)

    y = y if in_normalised_code_value else full_to_legal(y, bit_depth)

    x = np.where(
        y >= 171.2102946929 / 1023,
        ((10 ** ((y * 1023 - 420) / 261.5)) * (0.18 + 0.01) - 0.01),
        (y * 1023 - 95) * 0.01125000 / (171.2102946929 - 95),
    )

    if not out_reflection:
        x = x / 0.9

    return as_float(from_range_1(x))



"""
Common Colour Models Utilities
==============================

Defines various colour models common utilities:

-   :func:`colour.models.Jab_to_JCh`
-   :func:`colour.models.JCh_to_Jab`
-   :attr:`colour.COLOURSPACE_MODELS`

References
----------
-   :cite:`CIETC1-482004m` : CIE TC 1-48. (2004). CIE 1976 uniform colour
    spaces. In CIE 015:2004 Colorimetry, 3rd Edition (p. 24).
    ISBN:978-3-901906-33-6
"""

from __future__ import annotations

import numpy as np

from colour.algebra import cartesian_to_polar, polar_to_cartesian
from colour.hints import ArrayLike, NDArray, Tuple
from colour.utilities import (
    CaseInsensitiveMapping,
    attest,
    from_range_degrees,
    to_domain_degrees,
    tsplit,
    tstack,
)
from colour.utilities.documentation import (
    DocstringTuple,
    is_documentation_building,
)

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "Jab_to_JCh",
    "JCh_to_Jab",
    "COLOURSPACE_MODELS",
    "COLOURSPACE_MODELS_AXIS_LABELS",
    "COLOURSPACE_MODELS_DOMAIN_RANGE_SCALE_1_TO_REFERENCE",
]


def Jab_to_JCh(Jab: ArrayLike) -> NDArray:
    """
    Convert from *Jab* colour representation to *JCh* colour representation.

    This definition is used to perform conversion from *CIE L\\*a\\*b\\**
    colourspace to *CIE L\\*C\\*Hab* colourspace and for other similar
    conversions. It implements a generic transformation from *Lightness*
    :math:`J`, :math:`a` and :math:`b` opponent colour dimensions to the
    correlates of *Lightness* :math:`J`, chroma :math:`C` and hue angle
    :math:`h`.

    Parameters
    ----------
    Jab
        *Jab* colour representation array.

    Returns
    -------
    :class:`numpy.ndarray`
        *JCh* colour representation array.

    Notes
    -----
    +------------+-----------------------+-----------------+
    | **Domain** | **Scale - Reference** | **Scale - 1**   |
    +============+=======================+=================+
    | ``Jab``    | ``J`` : [0, 100]      | ``J`` : [0, 1]  |
    |            |                       |                 |
    |            | ``a`` : [-100, 100]   | ``a`` : [-1, 1] |
    |            |                       |                 |
    |            | ``b`` : [-100, 100]   | ``b`` : [-1, 1] |
    +------------+-----------------------+-----------------+

    +------------+-----------------------+-----------------+
    | **Range**  | **Scale - Reference** | **Scale - 1**   |
    +============+=======================+=================+
    | ``JCh``    | ``J``  : [0, 100]     | ``J`` : [0, 1]  |
    |            |                       |                 |
    |            | ``C``  : [0, 100]     | ``C`` : [0, 1]  |
    |            |                       |                 |
    |            | ``h`` : [0, 360]      | ``h`` : [0, 1]  |
    +------------+-----------------------+-----------------+

    References
    ----------
    :cite:`CIETC1-482004m`

    Examples
    --------
    >>> Jab = np.array([41.52787529, 52.63858304, 26.92317922])
    >>> Jab_to_JCh(Jab)  # doctest: +ELLIPSIS
    array([ 41.5278752...,  59.1242590...,  27.0884878...])
    """

    L, a, b = tsplit(Jab)

    C, H = tsplit(cartesian_to_polar(tstack([a, b])))

    JCh = tstack([L, C, from_range_degrees(np.degrees(H) % 360)])

    return JCh


def JCh_to_Jab(JCh: ArrayLike) -> NDArray:
    """
    Convert from *JCh* colour representation to *Jab* colour representation.

    This definition is used to perform conversion from *CIE L\\*C\\*Hab*
    colourspace to *CIE L\\*a\\*b\\** colourspace and for other similar
    conversions. It implements a generic transformation from the correlates of
    *Lightness* :math:`J`, chroma :math:`C` and hue angle :math:`h` to
    *Lightness* :math:`J`, :math:`a` and :math:`b` opponent colour dimensions.

    Parameters
    ----------
    JCh
        *JCh* colour representation array.

    Returns
    -------
    :class:`numpy.ndarray`
        *Jab* colour representation array.

    Notes
    -----
    +-------------+-----------------------+-----------------+
    | **Domain**  | **Scale - Reference** | **Scale - 1**   |
    +=============+=======================+=================+
    | ``JCh``     | ``J``  : [0, 100]     | ``J``  : [0, 1] |
    |             |                       |                 |
    |             | ``C``  : [0, 100]     | ``C``  : [0, 1] |
    |             |                       |                 |
    |             | ``h`` : [0, 360]      | ``h`` : [0, 1]  |
    +-------------+-----------------------+-----------------+

    +-------------+-----------------------+-----------------+
    | **Range**   | **Scale - Reference** | **Scale - 1**   |
    +=============+=======================+=================+
    | ``Jab``     | ``J`` : [0, 100]      | ``J`` : [0, 1]  |
    |             |                       |                 |
    |             | ``a`` : [-100, 100]   | ``a`` : [-1, 1] |
    |             |                       |                 |
    |             | ``b`` : [-100, 100]   | ``b`` : [-1, 1] |
    +-------------+-----------------------+-----------------+

    References
    ----------
    :cite:`CIETC1-482004m`

    Examples
    --------
    >>> JCh = np.array([41.52787529, 59.12425901, 27.08848784])
    >>> JCh_to_Jab(JCh)  # doctest: +ELLIPSIS
    array([ 41.5278752...,  52.6385830...,  26.9231792...])
    """

    L, C, H = tsplit(JCh)

    a, b = tsplit(
        polar_to_cartesian(tstack([C, np.radians(to_domain_degrees(H))]))
    )

    Jab = tstack([L, a, b])

    return Jab


COLOURSPACE_MODELS: Tuple = (
    "CAM02LCD",
    "CAM02SCD",
    "CAM02UCS",
    "CAM16LCD",
    "CAM16SCD",
    "CAM16UCS",
    "CIE XYZ",
    "CIE xyY",
    "CIE Lab",
    "CIE Luv",
    "CIE UCS",
    "CIE UVW",
    "DIN99",
    "Hunter Lab",
    "Hunter Rdab",
    "ICaCb",
    "ICtCp",
    "IPT",
    "IgPgTg",
    "Jzazbz",
    "OSA UCS",
    "Oklab",
    "hdr-CIELAB",
    "hdr-IPT",
)
if is_documentation_building():  # pragma: no cover
    COLOURSPACE_MODELS = DocstringTuple(COLOURSPACE_MODELS)
    COLOURSPACE_MODELS.__doc__ = """
Colourspace models supporting a direct conversion to *CIE XYZ* tristimulus
values.
"""

COLOURSPACE_MODELS_AXIS_LABELS: CaseInsensitiveMapping = (
    CaseInsensitiveMapping(
        {
            "CAM02LCD": ("$J^\\prime$", "$a^\\prime$", "$b^\\prime$"),
            "CAM02SCD": ("$J^\\prime$", "$a^\\prime$", "$b^\\prime$"),
            "CAM02UCS": ("$J^\\prime$", "$a^\\prime$", "$b^\\prime$"),
            "CAM16LCD": ("$J^\\prime$", "$a^\\prime$", "$b^\\prime$"),
            "CAM16SCD": ("$J^\\prime$", "$a^\\prime$", "$b^\\prime$"),
            "CAM16UCS": ("$J^\\prime$", "$a^\\prime$", "$b^\\prime$"),
            "CIE XYZ": ("X", "Y", "Z"),
            "CIE xyY": ("x", "y", "Y"),
            "CIE Lab": ("$L^*$", "$a^*$", "$b^*$"),
            "CIE Luv": ("$L^*$", "$u^\\prime$", "$v^\\prime$"),
            "CIE UCS": ("U", "V", "W"),
            "CIE UVW": ("U", "V", "W"),
            "DIN99": ("$L_{99}$", "$a_{99}$", "$b_{99}$"),
            "Hunter Lab": ("$L^*$", "$a^*$", "$b^*$"),
            "Hunter Rdab": ("Rd", "a", "b"),
            "ICaCb": ("$I$", "$C_a$", "$C_b$"),
            "ICtCp": ("$I$", "$C_T$", "$C_P$"),
            "IPT": ("I", "P", "T"),
            "IgPgTg": ("$I_G$", "$P_G$", "$T_G$"),
            "Jzazbz": ("$J_z$", "$a_z$", "$b_z$"),
            "OSA UCS": ("L", "j", "g"),
            "Oklab": ("$L$", "$a$", "$b$"),
            "hdr-CIELAB": ("L hdr", "a hdr", "b hdr"),
            "hdr-IPT": ("I hdr", "P hdr", "T hdr"),
        }
    )
)
"""Colourspace models labels mapping."""

attest(COLOURSPACE_MODELS == tuple(COLOURSPACE_MODELS_AXIS_LABELS.keys()))

COLOURSPACE_MODELS_DOMAIN_RANGE_SCALE_1_TO_REFERENCE: (
    CaseInsensitiveMapping
) = CaseInsensitiveMapping(
    {
        "CAM02LCD": np.array([100, 100, 100]),
        "CAM02SCD": np.array([100, 100, 100]),
        "CAM02UCS": np.array([100, 100, 100]),
        "CAM16LCD": np.array([100, 100, 100]),
        "CAM16SCD": np.array([100, 100, 100]),
        "CAM16UCS": np.array([100, 100, 100]),
        "CIE XYZ": np.array([1, 1, 1]),
        "CIE xyY": np.array([1, 1, 1]),
        "CIE Lab": np.array([100, 100, 100]),
        "CIE Luv": np.array([100, 100, 100]),
        "CIE UCS": np.array([1, 1, 1]),
        "CIE UVW": np.array([100, 100, 100]),
        "DIN99": np.array([100, 100, 100]),
        "Hunter Lab": np.array([100, 100, 100]),
        "Hunter Rdab": np.array([100, 100, 100]),
        "ICaCb": np.array([1, 1, 1]),
        "ICtCp": np.array([1, 1, 1]),
        "IPT": np.array([1, 1, 1]),
        "IgPgTg": np.array([1, 1, 1]),
        "Jzazbz": np.array([1, 1, 1]),
        "OSA UCS": np.array([100, 100, 100]),
        "Oklab": np.array([1, 1, 1]),
        "hdr-CIELAB": np.array([100, 100, 100]),
        "hdr-IPT": np.array([100, 100, 100]),
    }
)
"""Colourspace models domain-range scale **'1'** to **'Reference'** mapping."""



"""
Apple RGB Colourspace
=====================

Defines the *Apple RGB* colourspace:

-   :attr:`colour.models.RGB_COLOURSPACE_APPLE_RGB`.

References
----------
-   :cite:`Susstrunk1999a` : Susstrunk, S., Buckley, R., & Swen, S. (1999).
    Standard RGB Color Spaces.
"""

from __future__ import annotations

import numpy as np
from functools import partial

from colour.colorimetry import CCS_ILLUMINANTS
from colour.hints import NDArray
from colour.models.rgb import (
    RGB_Colourspace,
    gamma_function,
    normalised_primary_matrix,
)

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "PRIMARIES_APPLE_RGB",
    "WHITEPOINT_NAME_APPLE_RGB",
    "CCS_WHITEPOINT_APPLE_RGB",
    "MATRIX_APPLE_RGB_TO_XYZ",
    "MATRIX_XYZ_TO_APPLE_RGB",
    "RGB_COLOURSPACE_APPLE_RGB",
]

PRIMARIES_APPLE_RGB: NDArray = np.array(
    [
        [0.6250, 0.3400],
        [0.2800, 0.5950],
        [0.1550, 0.0700],
    ]
)
"""*Apple RGB* colourspace primaries."""

WHITEPOINT_NAME_APPLE_RGB: str = "D65"
"""*Apple RGB* colourspace whitepoint name."""

CCS_WHITEPOINT_APPLE_RGB: NDArray = CCS_ILLUMINANTS[
    "CIE 1931 2 Degree Standard Observer"
][WHITEPOINT_NAME_APPLE_RGB]
"""*Apple RGB* colourspace whitepoint chromaticity coordinates."""

MATRIX_APPLE_RGB_TO_XYZ: NDArray = normalised_primary_matrix(
    PRIMARIES_APPLE_RGB, CCS_WHITEPOINT_APPLE_RGB
)
"""*Apple RGB* colourspace to *CIE XYZ* tristimulus values matrix."""

MATRIX_XYZ_TO_APPLE_RGB: NDArray = np.linalg.inv(MATRIX_APPLE_RGB_TO_XYZ)
"""*CIE XYZ* tristimulus values to *Apple RGB* colourspace matrix."""

RGB_COLOURSPACE_APPLE_RGB: RGB_Colourspace = RGB_Colourspace(
    "Apple RGB",
    PRIMARIES_APPLE_RGB,
    CCS_WHITEPOINT_APPLE_RGB,
    WHITEPOINT_NAME_APPLE_RGB,
    MATRIX_APPLE_RGB_TO_XYZ,
    MATRIX_XYZ_TO_APPLE_RGB,
    partial(gamma_function, exponent=1 / 1.8),
    partial(gamma_function, exponent=1.8),
)
RGB_COLOURSPACE_APPLE_RGB.__doc__ = """
*Apple RGB* colourspace.

References
----------
:cite:`Susstrunk1999a`
"""
